# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgLastCurrent.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgLastCurrent(object):
    def setupUi(self, wdgLastCurrent):
        wdgLastCurrent.setObjectName("wdgLastCurrent")
        wdgLastCurrent.resize(750, 524)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(wdgLastCurrent)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(wdgLastCurrent)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.label = QtWidgets.QLabel(wdgLastCurrent)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spin = QtWidgets.QSpinBox(wdgLastCurrent)
        self.spin.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.spin.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.spin.setKeyboardTracking(False)
        self.spin.setMinimum(-100)
        self.spin.setMaximum(100)
        self.spin.setSingleStep(5)
        self.spin.setProperty("value", -25)
        self.spin.setObjectName("spin")
        self.horizontalLayout.addWidget(self.spin)
        self.cmd = QtWidgets.QToolButton(wdgLastCurrent)
        self.cmd.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/button_ok.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmd.setIcon(icon)
        self.cmd.setObjectName("cmd")
        self.horizontalLayout.addWidget(self.cmd)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.tblInvestments = myQTableWidget(wdgLastCurrent)
        self.tblInvestments.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblInvestments.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblInvestments.setAlternatingRowColors(True)
        self.tblInvestments.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblInvestments.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblInvestments.setObjectName("tblInvestments")
        self.tblInvestments.setColumnCount(9)
        self.tblInvestments.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(8, item)
        self.tblInvestments.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.tblInvestments)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.label_2 = QtWidgets.QLabel(wdgLastCurrent)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.cmbSameProduct = QtWidgets.QComboBox(wdgLastCurrent)
        self.cmbSameProduct.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.cmbSameProduct.setObjectName("cmbSameProduct")
        self.cmbSameProduct.addItem("")
        self.cmbSameProduct.addItem("")
        self.cmbSameProduct.addItem("")
        self.horizontalLayout_2.addWidget(self.cmbSameProduct)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.wdgIBM = QtWidgets.QWidget(wdgLastCurrent)
        self.wdgIBM.setObjectName("wdgIBM")
        self.verticalLayout.addWidget(self.wdgIBM)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.actionInvestmentReport = QtWidgets.QAction(wdgLastCurrent)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/bundle.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionInvestmentReport.setIcon(icon1)
        self.actionInvestmentReport.setObjectName("actionInvestmentReport")
        self.actionSortTPCVenta = QtWidgets.QAction(wdgLastCurrent)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpmoney/document-preview.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSortTPCVenta.setIcon(icon2)
        self.actionSortTPCVenta.setObjectName("actionSortTPCVenta")
        self.actionSortTPC = QtWidgets.QAction(wdgLastCurrent)
        self.actionSortTPC.setIcon(icon2)
        self.actionSortTPC.setObjectName("actionSortTPC")
        self.actionSortName = QtWidgets.QAction(wdgLastCurrent)
        self.actionSortName.setIcon(icon2)
        self.actionSortName.setObjectName("actionSortName")
        self.actionSortHour = QtWidgets.QAction(wdgLastCurrent)
        self.actionSortHour.setIcon(icon2)
        self.actionSortHour.setObjectName("actionSortHour")
        self.actionProduct = QtWidgets.QAction(wdgLastCurrent)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/books.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionProduct.setIcon(icon3)
        self.actionProduct.setObjectName("actionProduct")
        self.actionSortTPCLast = QtWidgets.QAction(wdgLastCurrent)
        self.actionSortTPCLast.setObjectName("actionSortTPCLast")
        self.actionCalculate = QtWidgets.QAction(wdgLastCurrent)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/xulpymoney/tools-wizard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionCalculate.setIcon(icon4)
        self.actionCalculate.setObjectName("actionCalculate")
        self.actionReinvest = QtWidgets.QAction(wdgLastCurrent)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/xulpymoney/coins.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionReinvest.setIcon(icon5)
        self.actionReinvest.setObjectName("actionReinvest")
        self.actionReinvestCurrent = QtWidgets.QAction(wdgLastCurrent)
        self.actionReinvestCurrent.setIcon(icon5)
        self.actionReinvestCurrent.setObjectName("actionReinvestCurrent")

        self.retranslateUi(wdgLastCurrent)
        QtCore.QMetaObject.connectSlotsByName(wdgLastCurrent)

    def retranslateUi(self, wdgLastCurrent):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgLastCurrent", "Last operation investments report"))
        self.label.setText(_translate("wdgLastCurrent", "Percentage gains got in last operation investment"))
        self.spin.setSuffix(_translate("wdgLastCurrent", " %"))
        item = self.tblInvestments.horizontalHeaderItem(0)
        item.setText(_translate("wdgLastCurrent", "Investment"))
        item = self.tblInvestments.horizontalHeaderItem(1)
        item.setText(_translate("wdgLastCurrent", "Last operation"))
        item = self.tblInvestments.horizontalHeaderItem(2)
        item.setText(_translate("wdgLastCurrent", "Last shares"))
        item = self.tblInvestments.horizontalHeaderItem(3)
        item.setText(_translate("wdgLastCurrent", "Total shares"))
        item = self.tblInvestments.horizontalHeaderItem(4)
        item.setText(_translate("wdgLastCurrent", "Balance"))
        item = self.tblInvestments.horizontalHeaderItem(5)
        item.setText(_translate("wdgLastCurrent", "Pending"))
        item = self.tblInvestments.horizontalHeaderItem(6)
        item.setText(_translate("wdgLastCurrent", "% Last"))
        item = self.tblInvestments.horizontalHeaderItem(7)
        item.setText(_translate("wdgLastCurrent", "% Invested"))
        item = self.tblInvestments.horizontalHeaderItem(8)
        item.setText(_translate("wdgLastCurrent", "% Selling point"))
        self.label_2.setText(_translate("wdgLastCurrent", "Select the way you want to see results"))
        self.cmbSameProduct.setItemText(0, _translate("wdgLastCurrent", "Show separated investments"))
        self.cmbSameProduct.setItemText(1, _translate("wdgLastCurrent", "Show merging current investment operations"))
        self.cmbSameProduct.setItemText(2, _translate("wdgLastCurrent", "Show merging all investment operations"))
        self.actionInvestmentReport.setText(_translate("wdgLastCurrent", "Investment report"))
        self.actionInvestmentReport.setToolTip(_translate("wdgLastCurrent", "Investment report"))
        self.actionSortTPCVenta.setText(_translate("wdgLastCurrent", "% Selling point"))
        self.actionSortTPCVenta.setToolTip(_translate("wdgLastCurrent", "% Selling point"))
        self.actionSortTPC.setText(_translate("wdgLastCurrent", "% Invested"))
        self.actionSortTPC.setToolTip(_translate("wdgLastCurrent", "% Invested"))
        self.actionSortName.setText(_translate("wdgLastCurrent", "Name"))
        self.actionSortName.setToolTip(_translate("wdgLastCurrent", "Name"))
        self.actionSortHour.setText(_translate("wdgLastCurrent", "Time"))
        self.actionSortHour.setToolTip(_translate("wdgLastCurrent", "Time"))
        self.actionProduct.setText(_translate("wdgLastCurrent", "Product report"))
        self.actionProduct.setToolTip(_translate("wdgLastCurrent", "Product report"))
        self.actionSortTPCLast.setText(_translate("wdgLastCurrent", "% Last operation"))
        self.actionSortTPCLast.setToolTip(_translate("wdgLastCurrent", "% Last operation"))
        self.actionCalculate.setText(_translate("wdgLastCurrent", "Calculate reinvest price"))
        self.actionCalculate.setToolTip(_translate("wdgLastCurrent", "Calculate reinvest price"))
        self.actionReinvest.setText(_translate("wdgLastCurrent", "Reinvest"))
        self.actionReinvest.setToolTip(_translate("wdgLastCurrent", "Reinvest"))
        self.actionReinvestCurrent.setText(_translate("wdgLastCurrent", "Simulate reinvestment at product current price"))
        self.actionReinvestCurrent.setToolTip(_translate("wdgLastCurrent", "Simulate reinvestment at product current price"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc



import datetime
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QMenu, QWidget, QTableWidgetItem, QHBoxLayout
from xulpymoney.libxulpymoney import Assets,  Percentage
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.libxulpymoneytypes import eQColor
from xulpymoney.ui.Ui_wdgConcepts import Ui_wdgConcepts
from xulpymoney.ui.wdgConceptsHistorical import wdgConceptsHistorical
from xulpymoney.ui.canvaschart import VCPie

class wdgConcepts(QWidget, Ui_wdgConcepts):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.selected=None
        
        self.viewIncomes=VCPie()
        self.layIncomes.addWidget(self.viewIncomes)
        self.viewExpenses=VCPie()
        self.layExpenses.addWidget(self.viewExpenses)
        self.expenses=self.mem.conceptos.clone_x_tipooperacion(1)
        self.expenseslist=None
        self.incomes=self.mem.conceptos.clone_x_tipooperacion(2)
        self.incomeslist=None

        self.tblExpenses.settings(self.mem, "wdgConcepts")
        self.tblIncomes.settings(self.mem, "wdgConcepts")
        
        anoinicio=Assets(self.mem).first_datetime_with_user_data().year       
        self.wdgYM.initiate(anoinicio,  datetime.date.today().year, datetime.date.today().year, datetime.date.today().month)
        self.wdgYM.changed.connect(self.on_wdgYM_changed)
        
        
        self.on_wdgYM_changed()
        
        self.tab.setCurrentIndex(0)
        
    def load_gastos(self,  year,  month):
        self.viewExpenses.clear()
        self.viewExpenses.chart.setTitle(self.tr("Concepts chart"))   
        self.viewExpenses.setCurrency(self.mem.localcurrency)
        
        (self.expenseslist, totalexpenses,  totalaverageexpenses)=self.expenses.percentage_monthly(year, month)
        self.tblExpenses.applySettings()
        self.tblExpenses.clearContents()
        self.tblExpenses.setRowCount(len(self.expenseslist)+1)
        
        for i, a in enumerate(self.expenseslist):
            self.tblExpenses.setItem(i, 0, QTableWidgetItem(a[0].name))
            self.tblExpenses.setItem(i, 1, self.mem.localcurrency.qtablewidgetitem(a[1]))
            self.tblExpenses.setItem(i, 2, Percentage(a[2], 100).qtablewidgetitem())#tpc
            self.tblExpenses.setItem(i, 3, self.mem.localcurrency.qtablewidgetitem(a[3]))
            
            if a[1]!=0:
                if a[1]>a[3]:
                    self.tblExpenses.item(i, 1).setBackground( eQColor.Green)          
                else:
                    self.tblExpenses.item(i, 1).setBackground( eQColor.Red)      
                self.viewExpenses.appendData(a[0].name.upper(), a[1])
        self.viewExpenses.display()
                
        self.tblExpenses.setItem(len(self.expenseslist), 0, QTableWidgetItem(self.tr('TOTAL')))
        self.tblExpenses.setItem(len(self.expenseslist), 1, self.mem.localcurrency.qtablewidgetitem(totalexpenses))    
        self.tblExpenses.setItem(len(self.expenseslist), 2, Percentage(1, 1).qtablewidgetitem())
        self.tblExpenses.setItem(len(self.expenseslist), 3, self.mem.localcurrency.qtablewidgetitem(totalaverageexpenses))       

    def load_ingresos(self,  year,  month):
        self.viewIncomes.clear()
        self.viewIncomes.chart.setTitle(self.tr("Concepts chart"))   
        self.viewIncomes.setCurrency(self.mem.localcurrency)
        (self.incomeslist, totalincomes,  totalaverageincomes)=self.incomes.percentage_monthly(year, month)
        self.tblIncomes.applySettings()
        self.tblIncomes.clearContents()
        self.tblIncomes.setRowCount(len(self.incomeslist)+1)
        
        for i, a in enumerate(self.incomeslist):
            self.tblIncomes.setItem(i, 0, QTableWidgetItem(a[0].name))
            self.tblIncomes.setItem(i, 1, self.mem.localcurrency.qtablewidgetitem(a[1]))
            self.tblIncomes.setItem(i, 2, Percentage(a[2], 100).qtablewidgetitem())#tpc
            self.tblIncomes.setItem(i, 3, self.mem.localcurrency.qtablewidgetitem(a[3]))
            
            if a[1]!=0:
                if a[1]>a[3]:
                    self.tblIncomes.item(i, 1).setBackground( eQColor.Green)          
                else:
                    self.tblIncomes.item(i, 1).setBackground( eQColor.Red)      
                self.viewIncomes.appendData(a[0].name.upper(), a[1])
        self.viewIncomes.display()
        self.tblIncomes.setItem(len(self.incomeslist), 0, QTableWidgetItem(self.tr('TOTAL')))
        self.tblIncomes.setItem(len(self.incomeslist), 1, self.mem.localcurrency.qtablewidgetitem(totalincomes))    
        self.tblIncomes.setItem(len(self.incomeslist), 2, Percentage(1, 1).qtablewidgetitem())
        self.tblIncomes.setItem(len(self.incomeslist), 3, self.mem.localcurrency.qtablewidgetitem(totalaverageincomes))         

    @pyqtSlot() 
    def on_wdgYM_changed(self):
        self.load_gastos(self.wdgYM.year, self.wdgYM.month)
        self.load_ingresos(self.wdgYM.year,  self.wdgYM.month)
        
    def on_tblExpenses_customContextMenuRequested(self, pos):
        self.selected=None
        for i in self.tblExpenses.selectedItems():#itera por cada item no row.
            if i.row()==len(self.expenseslist):#Si pulsa en total
                self.selected=None
                break
            if i.column()==0:
                self.selected=self.expenseslist[i.row()]

        
        if self.selected!=None:
            self.actionHistoricalReport.setEnabled(True)
        else:
            self.actionHistoricalReport.setEnabled(False)

        menu=QMenu()
        menu.addAction(self.actionHistoricalReport)   
        menu.exec_(self.tblExpenses.mapToGlobal(pos))
        
    def on_tblIncomes_customContextMenuRequested(self, pos):
        self.selected=None
        for i in self.tblIncomes.selectedItems():#itera por cada item no row.
            if i.row()==len(self.incomeslist):#Si pulsa en total
                self.selected=None
                break
            if i.column()==0:
                self.selected=self.incomeslist[i.row()]

        if self.selected!=None:
            self.actionHistoricalReport.setEnabled(True)
        else:
            self.actionHistoricalReport.setEnabled(False)
        
        menu=QMenu()
        menu.addAction(self.actionHistoricalReport)   
        menu.exec_(self.tblIncomes.mapToGlobal(pos))
        
    
    def on_tab_tabCloseRequested(self, index):
        """Only removes dinamic tabs"""
        if index in (0, 1):
            qmessagebox(self.tr("You can't close this tab"))
        else:
            self.tab.removeTab(index)
        
    @pyqtSlot()
    def on_actionHistoricalReport_triggered(self):
        newtab = QWidget()
        horizontalLayout = QHBoxLayout(newtab)
        concepto=self.selected[0]
        wch = wdgConceptsHistorical(self.mem, concepto, newtab)
        horizontalLayout.addWidget(wch)
        self.tab.addTab(newtab, "{0}".format(concepto.name))
        self.tab.setCurrentWidget(newtab)




## @namespace xulpymoney.ui.wdgProductHistoricalChart
## @brief Several product historical chart widgets

from PyQt5.QtCore import pyqtSlot, Qt,  QDate
from PyQt5.QtGui import QColor,  QPen,  QIcon, QPixmap,  QWheelEvent
from PyQt5.QtWidgets import QWidget,  QHBoxLayout, QLabel,  QToolButton,  QSpacerItem,  QSizePolicy,  QPushButton, QVBoxLayout, QDialog, QLineEdit
from xulpymoney.ui.Ui_wdgProductHistoricalChart import Ui_wdgProductHistoricalChart

import datetime
import logging
from decimal import Decimal
from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.canvaschart import   VCTemporalSeries
from xulpymoney.libxulpymoney import InvestmentOperation,  Investment,  Money, Percentage, InvestmentOperationHomogeneusManager
from xulpymoney.libxulpymoneyfunctions import day_start_from_date, day_start, string2list_of_integers
from xulpymoney.libxulpymoneytypes import eHistoricalChartAdjusts, eOHCLDuration,  eOperationType
from xulpymoney.ui.wdgOpportunitiesAdd import wdgOpportunitiesAdd

## Main class that sets a product (can add an investment too) with setProduct function
##
## OHCL set is updated each time generate function is called
class wdgProductHistoricalChart(QWidget, Ui_wdgProductHistoricalChart):
    def __init__(self,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.view=None
        self.dtFrom.blockSignals(True)
        self.HistoricalChartAdjusts=eHistoricalChartAdjusts.Splits
        
    def _pen(self, style, color):
        pen=QPen()
        pen.setStyle(style)
        pen.setColor(color)
        return pen

    def setProduct(self, product, investment=None):
        self.product=product
        self.investment=investment
        self.mem=self.product.mem
        from_=datetime.datetime.today()-datetime.timedelta(days=365*3)
        if self.investment!=None:
            if self.investment.op_actual.length()>0:
                from_=self.investment.op_actual.first().datetime-datetime.timedelta(days=30)
            elif self.investment.op.length()>0:
                from_=self.investment.op.first().datetime-datetime.timedelta(days=30)
        self.dtFrom.setDate(from_)
        self.dtFrom.blockSignals(False)
        self.cmbOHCLDuration.currentIndexChanged.disconnect()
        eOHCLDuration.qcombobox(self.cmbOHCLDuration, eOHCLDuration.Day)
        self.cmbOHCLDuration.currentIndexChanged.connect(self.on_cmbOHCLDuration_currentIndexChanged)
        
    @pyqtSlot(QDate) 
    def on_dtFrom_dateChanged(self, date):
        selected=date.toPyDate()
        if datetime.date.today()-selected<=datetime.timedelta(days=366):
            self.cmdFromRight.setEnabled(False)
        else:
            self.cmdFromRight.setEnabled(True)
        self.generate()
        self.display()

    ## Just draw the chart with selected options. It creates and destroys objects
    ##
    ## self.setohcl is set calling this function
    def generate(self):
        if self.view!=None:
            self.view.hide()
            self.view.close()
            self.verticalLayout.removeWidget(self.view)

        selected_datetime= day_start_from_date(self.dtFrom.date().toPyDate(), self.mem.localzone)
        self.setohcl=self.product.result.ohcl(self.cmbOHCLDuration.itemData(self.cmbOHCLDuration.currentIndex()), eHistoricalChartAdjusts.Splits)
        
        self.view=VCTemporalSeries()
        self.verticalLayout.addWidget(self.view)
        if self.cmbChartType.currentIndex()==0:#Lines
            ls=self.view.appendTemporalSeries(self.product.name.upper(), self.product.currency)#Line seies
            for ohcl in self.setohcl.arr:
                if ohcl.datetime()>=selected_datetime:
                    self.view.appendTemporalSeriesData(ls, day_start(ohcl.datetime(), self.mem.localzone), ohcl.close) #Added day_start to show the purchase circle the first day
        elif self.cmbChartType.currentIndex()==1:#Candles            
            candle=self.view.appendCandlestickSeries(self.product.name, self.product.currency)#Candle series
            for ohcl in self.setohcl.arr:
                if ohcl.datetime()>=selected_datetime-datetime.timedelta(days=10):#Added one day to show the purchase circle the first day
                    self.view.appendCandlestickSeriesData(candle, ohcl)
            self.view.setOHCLDuration(self.cmbOHCLDuration.itemData(self.cmbOHCLDuration.currentIndex()))
            
        if self.chkSMA50.isChecked() and self.setohcl.length()>50:#SMA50 line series
            sma50=self.view.appendTemporalSeries(self.tr("SMA50"),  self.product.currency)
            sma50.setColor(QColor(255, 170, 255))
            for dt, value in self.setohcl.sma(50):
                if dt>=selected_datetime:
                    self.view.appendTemporalSeriesData(sma50, dt, value)
                
        if self.chkSMA200.isChecked() and self.setohcl.length()>200:#SMA200 line series
            sma200=self.view.appendTemporalSeries(self.tr("SMA200"),  self.product.currency)
            sma200.setColor(QColor(165, 165, 165))
            for dt, value in self.setohcl.sma(200):
                if dt>=selected_datetime:
                    self.view.appendTemporalSeriesData(sma200, dt, value)

        if self.chkMedian.isChecked():#Median value
            median=self.setohcl.closes_median_value()
            med=self.view.appendTemporalSeries(self.tr("Median at {}".format(self.product.currency.string(median))),  self.product.currency)
            med.setColor(QColor(165, 165, 0))
            self.view.appendTemporalSeriesData(med, selected_datetime, median)
            self.view.appendTemporalSeriesData(med, self.mem.localzone.now(), median)

        if not self.chkAdjustSplits.isChecked():#
            ls=self.view.appendTemporalSeries(self.product.name.upper() + " (No adjust)", self.product.currency)#Line seies
            for ohcl in self.product.result.ohcl(self.cmbOHCLDuration.itemData(self.cmbOHCLDuration.currentIndex()), eHistoricalChartAdjusts.NoAdjusts).arr:
                if ohcl.datetime()>=selected_datetime:
                    self.view.appendTemporalSeriesData(ls, day_start(ohcl.datetime(), self.mem.localzone), ohcl.close) 

        if self.chkAdjustDividends.isChecked():#
            ls=self.view.appendTemporalSeries(self.product.name.upper() + (" (Dividend adjust)"), self.product.currency)#Line seies
            for ohcl in self.product.result.ohcl(self.cmbOHCLDuration.itemData(self.cmbOHCLDuration.currentIndex()), eHistoricalChartAdjusts.SplitsAndDividends).arr:
                if ohcl.datetime()>=selected_datetime:
                    self.view.appendTemporalSeriesData(ls, day_start(ohcl.datetime(), self.mem.localzone), ohcl.close) 

        #INVESTMENT
        if self.investment!=None:
            #Buy sell operations
            buy=self.view.appendScatterSeries(self.tr("Buy operations"), self.product.currency)
            buy.setColor(QColor(85, 170, 127))
            sell=self.view.appendScatterSeries(self.tr("Sell operations"), self.product.currency)
            sell.setColor(QColor(170, 85, 85))
            for op in self.investment.op.arr:
                if (    op.tipooperacion.id in (eOperationType.SharesAdd, eOperationType.SharesPurchase, eOperationType.TransferSharesDestiny ) or 
                        (op.tipooperacion.id==eOperationType.TransferFunds and op.shares>0)
                    ) and op.datetime>=selected_datetime:
                    self.view.appendScatterSeriesData(buy, op.datetime, op.valor_accion)
                if (    op.tipooperacion.id in (eOperationType.TransferSharesOrigin, eOperationType.SharesSale) or 
                        (op.tipooperacion.id==eOperationType.TransferFunds and op.shares<0)
                    ) and op.datetime>=selected_datetime:
                    self.view.appendScatterSeriesData(sell, op.datetime, op.valor_accion)
            
            #Average price
            if self.investment.op_actual.length()>0:
                m_selling_price=self.investment.selling_price()
                m_average_price=self.investment.op_actual.average_price(type=1)
                if self.investment.selling_expiration!=None:#If no selling point, it makes ugly the chart
                    selling_price=self.view.appendTemporalSeries(self.tr("Selling price at {} to gain {}".format(m_selling_price,  self.investment.op_actual.gains_in_selling_point())),  self.product.currency)
                    selling_price.setColor(QColor(170, 85, 85))
                    self.view.appendTemporalSeriesData(selling_price, self.investment.op_actual.first().datetime, m_selling_price.amount)
                    self.view.appendTemporalSeriesData(selling_price, self.mem.localzone.now(), m_selling_price.amount)
                average_price=self.view.appendTemporalSeries(self.tr("Average price at {}".format(m_average_price)),  self.product.currency)
                average_price.setColor(QColor(85, 170, 127))
                self.view.appendTemporalSeriesData(average_price, self.investment.op_actual.first().datetime, m_average_price.amount)
                self.view.appendTemporalSeriesData(average_price, self.mem.localzone.now(), m_average_price.amount)

    def display(self):
        self.view.display()
            
    @pyqtSlot(int)      
    def on_cmbChartType_currentIndexChanged(self, index):
        self.generate()
        self.display()

    @pyqtSlot(int)      
    def on_cmbOHCLDuration_currentIndexChanged(self, index):
        self.generate()
        self.display()
        
    def on_cmdFromRight_released(self):
        self.dtFrom.setDate(day_start_from_date(self.dtFrom.date().toPyDate(), self.mem.localzone)+datetime.timedelta(days=365))
        self.generate()
        self.display()        
        
    def on_cmdFromLeft_released(self):
        self.dtFrom.setDate(day_start_from_date(self.dtFrom.date().toPyDate(), self.mem.localzone)-datetime.timedelta(days=365))
        self.generate()
        self.display()        
        
    def on_cmdFromRightMax_released(self):
        self.dtFrom.setDate(self.setohcl.last().datetime()-datetime.timedelta(days=365))
        self.generate()
        self.display()        
        
    def on_cmdFromLeftMax_released(self):
        self.dtFrom.setDate(self.setohcl.first().datetime())
        self.generate()
        self.display()
        
    def on_chkSMA50_stateChanged(self, state):
        self.generate()
        self.display()
    
    def on_chkSMA200_stateChanged(self, state):
        self.generate()
        self.display()
    def on_chkAdjustSplits_stateChanged(self, state):
        self.generate()
        self.display()
    def on_chkAdjustDividends_stateChanged(self, state):
        self.generate()
        self.display()
    
    ## Function executed when changing state in QCheckbox chkmedian
    def on_chkMedian_stateChanged(self, state):
        self.generate()
        self.display()
        
    @pyqtSlot(QWheelEvent)
    def wheelEvent(self, event: QWheelEvent):
        def checkDate(newdate):
            """
                Check newdate doesn't go over bounds
            """
            date=self.dtFrom.date()
            firstdate=self.product.result.ohcl(self.cmbOHCLDuration.itemData(self.cmbOHCLDuration.currentIndex())).first().datetime().date()
            if newdate>date:
                if newdate>datetime.date.today():
                    return date
                else:
                    return newdate
            else:#newdate<=date
                if newdate<firstdate:
                    return date
                else:
                    return newdate
        ##############################  
        event.accept()#If I dont use this makes two events
        if event.modifiers() == Qt.ControlModifier:
            step=30
        elif event.modifiers() == Qt.ShiftModifier:
            step=7
        else:
            step=365

        if event.angleDelta().y()<0:
            self.dtFrom.setDate(checkDate(self.dtFrom.date().toPyDate()-datetime.timedelta(days=step)))
        else:
            self.dtFrom.setDate(checkDate(self.dtFrom.date().toPyDate()+datetime.timedelta(days=step)))


class wdgProductHistoricalReinvestChart(wdgProductHistoricalChart):
    def __init__(self,  parent=None):
        wdgProductHistoricalChart.__init__(self, parent)
        self.sim_op=None
        self.sim_opactual=None
        self.lblComment=QLabel()
        
    def setReinvest(self,  sim_op,  sim_opactual):
        self.sim_op=sim_op
        self.sim_opactual=sim_opactual
        

    def generate(self):
        """Just draw the chart with selected options. It creates and destroys objects"""
            
        wdgProductHistoricalChart.generate(self)
        if self.investment!=None:
            if self.investment.op_actual.length()>0:
                #Calcs
                percentage=Percentage(Decimal(self.mem.settingsdb.value("frmSellingPoint/lastgainpercentage",  5)), 100)
                new_avg_1=self.sim_opactual.average_price()
                new_sell_price_1=self.sim_opactual.average_price_after_a_gains_percentage(percentage)                
                new_purchase_price_1=self.sim_opactual.last().price()
                gains_1=self.sim_opactual.gains_from_percentage(percentage)
                
                #First reinvestment
                if self.investment.selling_expiration!=None:#If no selling point, it makes ugly the chart
                    new_selling_price=self.view.appendTemporalSeries(self.tr("Reinvestment selling price at {} to gain {}".format(new_sell_price_1, gains_1)),  self.product.currency)
                    new_selling_price.setColor(QColor(170, 85, 85))
                    new_selling_price.setPen(self._pen(Qt.DashLine, QColor(170, 85, 85)))
                    self.view.appendTemporalSeriesData(new_selling_price, self.investment.op_actual.first().datetime, new_sell_price_1.amount)
                    self.view.appendTemporalSeriesData(new_selling_price, self.mem.localzone.now(), new_sell_price_1.amount)
                
                new_average_price=self.view.appendTemporalSeries(self.tr("Reinvestment average price at {}").format(new_avg_1),  self.product.currency)
                new_average_price.setColor(QColor(85, 85, 170))
                new_average_price.setPen(self._pen(Qt.DashLine, QColor(85, 85, 170)))
                self.view.appendTemporalSeriesData(new_average_price, self.investment.op_actual.first().datetime, new_avg_1.amount)
                self.view.appendTemporalSeriesData(new_average_price, self.mem.localzone.now(), new_avg_1.amount)
                
                new_purchase_price=self.view.appendTemporalSeries(self.tr("Reinvestment purchase at {}").format(new_purchase_price_1),  self.product.currency)
                new_purchase_price.setColor(QColor(85, 170, 127))
                new_purchase_price.setPen(self._pen(Qt.DashLine, QColor(85, 170, 127)))
                self.view.appendTemporalSeriesData(new_purchase_price, self.sim_opactual.first().datetime, new_purchase_price_1.amount)
                self.view.appendTemporalSeriesData(new_purchase_price, self.mem.localzone.now(), new_purchase_price_1.amount) 
                
                self.lblComment.setText(self.tr("Gains percentage: {}".format(percentage.string())))
                self.verticalLayout.addWidget(self.lblComment)

class ReinvestmentLines:
    Buy=0
    Average=1
    Sell=2


class wdgProductHistoricalBuyChart(wdgProductHistoricalChart):
    def __init__(self,  parent=None):
        wdgProductHistoricalChart.__init__(self, parent)
        self.labelBuyPrice=QLabel(self.tr("Add wanted price"))
        self.txtBuyPrice=myQLineEdit(self)                                                                                                    
        self.cmdBuyPrice= QToolButton(self)               
        icon = QIcon()
        icon.addPixmap(QPixmap(":/xulpymoney/tools-wizard.png"), QIcon.Normal, QIcon.Off)
        self.cmdBuyPrice.setIcon(icon)                                      
        self.horizontalLayout_3.addWidget(self.labelBuyPrice)
        self.spacerBuyPrice = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addWidget(self.txtBuyPrice)
        self.horizontalLayout_3.addWidget(self.cmdBuyPrice)
        self.horizontalLayout_3.addItem(self.spacerBuyPrice)
        
        self.cmdBuyPrice.released.connect(self.on_cmdBuyPrice_released)
        
        self.layAmounts = QHBoxLayout()
        self.label1=QLabel(self)
        self.label1.setText(self.tr("Amounts to invest separated by ;"))
        self.txtAmounts=QLineEdit(self)
        self.txtAmounts.setText("2500;3500;8400;8400")
        
        self.labelGains=QLabel(self)
        self.labelGains.setText(self.tr("Gains percentage"))
        self.txtGains=myQLineEdit(self)
        self.txtGains.setText(10)
        
        
        self.labelLastOperationPercentage=QLabel(self)
        self.labelLastOperationPercentage.setText(self.tr("Last Operation next buy percentage"))
        self.txtLastOperationPercentage=myQLineEdit(self)
        self.txtLastOperationPercentage.setText(33)
        
        self.spacer1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.spacer2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.spacer3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.spacer4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.spacer5 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.spacer6 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.layAmounts.addItem(self.spacer1)
        self.layAmounts.addWidget(self.label1)
        self.layAmounts.addWidget(self.txtAmounts)
        self.layAmounts.addItem(self.spacer2)
        self.layAmounts.addWidget(self.labelGains)
        self.layAmounts.addWidget(self.txtGains)
        self.layAmounts.addItem(self.spacer5)
        self.layAmounts.addWidget(self.labelLastOperationPercentage)
        self.layAmounts.addWidget(self.txtLastOperationPercentage)
        self.layAmounts.addItem(self.spacer6)
        self.verticalLayout.addLayout(self.layAmounts)
        
        #Add Calculator widget
        self.layOpportunity=QHBoxLayout()
        self.spacerOpportunity=QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.cmdOpportunity=QPushButton(self.tr("Add Opportunity"))
        self.cmdOpportunity.released.connect(self.on_cmdOpportunity_released)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap(":/xulpymoney/kcalc.png"), QIcon.Normal, QIcon.Off)
        self.cmdOpportunity.setIcon(icon1)
        self.layOpportunity.addItem(self.spacerOpportunity)
        self.layOpportunity.addWidget(self.cmdOpportunity)
        
        
        
    ## Must be added after setProduct
    def setPrice(self, price):
        self.txtBuyPrice.setText(price)
        
    def on_cmdBuyPrice_released(self):
        if self.txtBuyPrice.isValid():
            self.generate()
            self.display()
    
    ## This button allows to open order Opportunity with the first purchase price
    def on_cmdOpportunity_released(self):
        d=QDialog(self)     
        d.setModal(True)
        d.setWindowTitle(self.tr("Add new opportunity"))
        w=wdgOpportunitiesAdd(self.mem, None, d)
        w.productSelector.setSelected(self.product)
        w.txtPrice.setText(self.txtBuyPrice.decimal())
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()    
        
    def __qcolor_by_reinvestment_line(self, reinvestment_line):
        if reinvestment_line==ReinvestmentLines.Buy:
            return QColor(85, 170, 127)
        elif reinvestment_line==ReinvestmentLines.Average:
            return QColor(85, 85, 170)
        elif reinvestment_line==ReinvestmentLines.Sell:
            return QColor(170, 85, 85)
            
    def __qpen_by_amounts_index(self, index, reinvestment_line):
        if index==0:
            return self._pen(Qt.SolidLine, self.__qcolor_by_reinvestment_line(reinvestment_line))
        elif index==1:
            return self._pen(Qt.DashLine, self.__qcolor_by_reinvestment_line(reinvestment_line))
        elif index==2:
            return self._pen(Qt.DotLine, self.__qcolor_by_reinvestment_line(reinvestment_line))
        else:
            return self._pen(Qt.DotLine, self.__qcolor_by_reinvestment_line(reinvestment_line))

    ## Just draw the chart with selected options. It creates and destroys objects
    def generate(self):
        self.amounts=string2list_of_integers(self.txtAmounts.text(), ";")
        logging.debug(self.amounts)
        wdgProductHistoricalChart.generate(self)
        
        percentage=Percentage(self.txtGains.decimal(), 100)
        selected_datetime= day_start_from_date(self.dtFrom.date().toPyDate(), self.mem.localzone)

        inv=Investment(self.mem).init__create("Buy Chart", None, None, self.product, None, True, -1)
        inv.op=InvestmentOperationHomogeneusManager(self.mem, inv)
        p_last_operation=Percentage(self.txtLastOperationPercentage.decimal(), 100)

        m_purchase=Money(self.mem, self.txtBuyPrice.decimal(), self.product.currency)
        #index=0 purchase, 1 = first reinvestment
        lastIO=None
        for index, amount in enumerate(self.amounts):
            lastIO=InvestmentOperation(self.mem).init__create  (   self.mem.tiposoperaciones.find_by_id(4), 
                                                                                            self.mem.localzone.now(), 
                                                                                            inv, 
                                                                                            int(self.amounts[index]/m_purchase.amount), 
                                                                                            0, 
                                                                                            0, 
                                                                                            m_purchase.amount, 
                                                                                            "",  
                                                                                            True, 
                                                                                            1,  
                                                                                            -10000
                                                                                        )
            inv.op.append(lastIO)
            (inv.op_actual, inv.op_historica)=inv.op.get_current_and_historical_operations()
            new_purchase_price=self.view.appendTemporalSeries(self.tr("Purchase price {}: {}").format(index, m_purchase.string()),  self.product.currency)
            new_purchase_price.setColor(self.__qcolor_by_reinvestment_line(ReinvestmentLines.Buy))
            new_purchase_price.setPen(self.__qpen_by_amounts_index(index, ReinvestmentLines.Buy))
            self.view.appendTemporalSeriesData(new_purchase_price, selected_datetime, m_purchase.amount)
            self.view.appendTemporalSeriesData(new_purchase_price, self.mem.localzone.now(), m_purchase.amount)
        
            m_new_average_price=inv.op_actual.average_price()
            m_new_selling_price=inv.op_actual.average_price_after_a_gains_percentage(percentage)
            new_average_price=self.view.appendTemporalSeries(self.tr("Average price {}: {}".format(index, m_new_average_price)),  self.product.currency)
            new_average_price.setColor(self.__qcolor_by_reinvestment_line(ReinvestmentLines.Average))
            new_average_price.setPen(self.__qpen_by_amounts_index(index, ReinvestmentLines.Average))
            self.view.appendTemporalSeriesData(new_average_price, selected_datetime, m_new_average_price.amount)
            self.view.appendTemporalSeriesData(new_average_price, self.mem.localzone.now(), m_new_average_price.amount)

            new_selling_price=self.view.appendTemporalSeries(self.tr("Sell price {} at {} to gain {}".format(index, m_new_selling_price, inv.op_actual.gains_from_percentage(percentage))),  self.product.currency)
            new_selling_price.setColor(self.__qcolor_by_reinvestment_line(ReinvestmentLines.Sell))
            new_selling_price.setPen(self.__qpen_by_amounts_index(index, ReinvestmentLines.Sell))
            self.view.appendTemporalSeriesData(new_selling_price, selected_datetime, m_new_selling_price.amount)
            self.view.appendTemporalSeriesData(new_selling_price, self.mem.localzone.now(),m_new_selling_price.amount)

#            buyprice=lastIO.valor_accion*(1-p_last_operation.value)
            m_purchase=Money(self.mem, lastIO.valor_accion*(1-p_last_operation.value),  self.product.currency)

#        #FIRST REINVESTMENT
#        p_last_operation=Percentage(self.txtLastOperationPercentage.decimal(), 100)
#        m_r1_purchase=Money(self.mem, d1.valor_accion*(1-p_last_operation.value),  self.product.currency)
#        d2=InvestmentOperation(self.mem).init__create  (   self.mem.tiposoperaciones.find_by_id(4), 
#                                                                                        self.mem.localzone.now(), 
#                                                                                        inv, 
#                                                                                        int(self.txt2.decimal()/m_r1_purchase.amount), 
#                                                                                        0, 
#                                                                                        0, 
#                                                                                        m_r1_purchase.amount, 
#                                                                                        "",  
#                                                                                        True, 
#                                                                                        1,  
#                                                                                        -9999
#                                                                                    )
#        inv.op.append(d2)
#        (inv.op_actual, inv.op_historica)=inv.op.get_current_and_historical_operations()
#        new_purchase_price=self.view.appendTemporalSeries(self.tr("First reinvestment purchase: {}").format(m_r1_purchase.string()),  self.product.currency)
#        new_purchase_price.setColor(QColor(85, 170, 127))
#        new_purchase_price.setPen(self._pen(Qt.DashLine, QColor(85, 170, 127)))
#        self.view.appendTemporalSeriesData(new_purchase_price, selected_datetime, m_r1_purchase.amount)
#        self.view.appendTemporalSeriesData(new_purchase_price, self.mem.localzone.now(), m_r1_purchase.amount)
#        
#        m_r1_average=inv.op_actual.average_price()
#        m_r1_sell=inv.op_actual.average_price_after_a_gains_percentage(percentage)
#        new_average_price=self.view.appendTemporalSeries(self.tr("First reinvestment average price: {}").format(m_r1_average.string()),  self.product.currency)
#        new_average_price.setColor(QColor(85, 85, 170))
#        new_average_price.setPen(self._pen(Qt.DashLine, QColor(85, 85, 170)))
#        self.view.appendTemporalSeriesData(new_average_price, selected_datetime, m_r1_average.amount)
#        self.view.appendTemporalSeriesData(new_average_price, self.mem.localzone.now(), m_r1_average.amount)
#
#        new_selling_price=self.view.appendTemporalSeries(self.tr("First reinvestment sale price at {} to gain {}".format(m_r1_sell, inv.op_actual.gains_from_percentage(percentage))),   self.product.currency)
#        new_selling_price.setColor(QColor(170, 85, 85))
#        new_selling_price.setPen(self._pen(Qt.DashLine, QColor(170, 85, 85)))
#        self.view.appendTemporalSeriesData(new_selling_price, selected_datetime, m_r1_sell.amount)
#        self.view.appendTemporalSeriesData(new_selling_price, self.mem.localzone.now(),  m_r1_sell.amount)
#        
#        self.verticalLayout.addLayout(self.layOpportunity)
#        
#
#        #SECOND REINVESTMENT
#        m_r2_purchase=Money(self.mem, d2.valor_accion*(1-p_last_operation.value),  self.product.currency)
#        d3=InvestmentOperation(self.mem).init__create  (   self.mem.tiposoperaciones.find_by_id(4), 
#                                                                                        self.mem.localzone.now(), 
#                                                                                        inv, 
#                                                                                        int(self.txt3.decimal()/m_r2_purchase.amount), 
#                                                                                        0, 
#                                                                                        0, 
#                                                                                        m_r2_purchase.amount, 
#                                                                                        "",  
#                                                                                        True, 
#                                                                                        1,  
#                                                                                        -9998
#                                                                                        )
#        inv.op.append(d3)
#        (inv.op_actual, inv.op_historica)=inv.op.get_current_and_historical_operations()
#        new_purchase_price=self.view.appendTemporalSeries(self.tr("Second reinvestment purchase: {}").format(m_r2_purchase),  self.product.currency)
#        new_purchase_price.setColor(QColor(85, 170, 127))
#        new_purchase_price.setPen(self._pen(Qt.DotLine, QColor(85, 170, 127)))
#        self.view.appendTemporalSeriesData(new_purchase_price, selected_datetime, m_r2_purchase.amount)
#        self.view.appendTemporalSeriesData(new_purchase_price, self.mem.localzone.now(), m_r2_purchase.amount)
#        
#        m_r2_average=inv.op_actual.average_price()
#        m_r2_sell=inv.op_actual.average_price_after_a_gains_percentage(percentage)
#        new_average_price=self.view.appendTemporalSeries(self.tr("Second reinvestment average price: {}").format(m_r2_average),  self.product.currency)
#        new_average_price.setColor(QColor(85, 85, 170))
#        new_average_price.setPen(self._pen(Qt.DotLine, QColor(85, 85, 170)))
#        self.view.appendTemporalSeriesData(new_average_price, selected_datetime, m_r2_average.amount)
#        self.view.appendTemporalSeriesData(new_average_price, self.mem.localzone.now(), m_r2_average.amount)
#
#        new_selling_price=self.view.appendTemporalSeries(self.tr("Second reinvestment sale price at {} to gain {}".format(m_r2_sell, inv.op_actual.gains_from_percentage(percentage))),   self.product.currency)
#        new_selling_price.setColor(QColor(170, 85, 85))
#        new_selling_price.setPen(self._pen(Qt.DotLine, QColor(170, 85, 85)))
#        self.view.appendTemporalSeriesData(new_selling_price, selected_datetime, m_r2_sell.amount)
#        self.view.appendTemporalSeriesData(new_selling_price, self.mem.localzone.now(),  m_r2_sell.amount)




class wdgProductHistoricalOpportunity(wdgProductHistoricalChart):
    def __init__(self,  parent=None):
        wdgProductHistoricalChart.__init__(self, parent)
    
    def setOpportunity(self,  opportunity):
        self.opportunity=opportunity
        
    def generate(self):
        wdgProductHistoricalChart.generate(self)
        if not hasattr(self, 'opportunity'):
            logging.debug(self.tr("You need to use wdgProductHistoricalOpportunity.setOpportunity before wdgProductHistoricalOpportunity.generate"))
            return
            
        if self.opportunity.entry!=None:
            entry=self.view.appendTemporalSeries(self.tr("Entry"),  self.product.currency)
            entry.setColor(QColor(85, 85, 170))
            entry.setPen(self._pen(Qt.DashLine, QColor(85, 85, 170)))
            self.view.appendTemporalSeriesData(entry, day_start_from_date(self.dtFrom.date().toPyDate(), self.mem.localzone)-datetime.timedelta(days=10), self.opportunity.entry)
            self.view.appendTemporalSeriesData(entry, self.mem.localzone.now()+datetime.timedelta(days=10), self.opportunity.entry)

        if self.opportunity.target!=None:
            target=self.view.appendTemporalSeries(self.tr("Target"),  self.product.currency)
            target.setColor(QColor(85, 170, 85))
            target.setPen(self._pen(Qt.DashLine, QColor(85, 170, 85)))
            self.view.appendTemporalSeriesData(target, day_start_from_date(self.dtFrom.date().toPyDate(), self.mem.localzone)-datetime.timedelta(days=10), self.opportunity.target)
            self.view.appendTemporalSeriesData(target, self.mem.localzone.now()+datetime.timedelta(days=10), self.opportunity.target)

        if self.opportunity.stoploss!=None:
            stoploss=self.view.appendTemporalSeries(self.tr("Stop loss"),  self.product.currency)
            stoploss.setColor(QColor(170, 85, 85))
            stoploss.setPen(self._pen(Qt.DashLine, QColor(170, 85, 85)))
            self.view.appendTemporalSeriesData(stoploss, day_start_from_date(self.dtFrom.date().toPyDate(), self.mem.localzone)-datetime.timedelta(days=10), self.opportunity.stoploss)
            self.view.appendTemporalSeriesData(stoploss, self.mem.localzone.now()+datetime.timedelta(days=10), self.opportunity.stoploss)

                



## @brief Package to manage postgresql connection functionss
## THIS IS FROM XULPYMONEY PACKAGE IF YOU NEED THIS MODULE PLEASE SYNC IT FROM THERE, FOR EXAMPLE
## @code
##       print ("Copying libmanagers.py from Xulpymoney project")
##        os.chdir("your directory)
##        os.remove("connection_pg.py")
##        os.system("wget https://raw.githubusercontent.com/Turulomio/xulpymoney/master/xulpymoney/connection_pg.py  --no-clobber")
##        os.system("sed -i -e '3i ## THIS FILE HAS BEEN DOWNLOADED AT {} FROM https://github.com/Turulomio/xulpymoney/xulpymoney/connection_pg.py.' connection_pg.py".format(datetime.datetime.now()))
## @encode

import datetime
from psycopg2 import OperationalError
from psycopg2.extras import DictConnection

class Connection:
    def __init__(self):
        self.user=None
        self.password=None
        self.server=None
        self.port=None
        self.db=None
        self._con=None
        self.init=None

    def init__create(self, user, password, server, port, db):
        self.user=user
        self.password=password
        self.server=server
        self.port=port
        self.db=db
        return self

    def cursor(self):
        return self._con.cursor()

    def mogrify(self, sql, arr):
        cur=self._con.cursor()
        s=cur.mogrify(sql, arr)
        cur.close()
        return  s

    def setAutocommit(self, b):
        self._con.autocommit = b


    def cursor_one_row(self, sql, arr=[]):
        cur=self._con.cursor()
        cur.execute(sql, arr)
        row=cur.fetchone()
        cur.close()
        return row

    def cursor_rows(self, sql, arr=[]):
        cur=self._con.cursor()
        cur.execute(sql, arr)
        rows=cur.fetchall()
        cur.close()
        return rows

    def load_script(self, file):
        cur= self._con.cursor()
        procedures  = open(file,'r').read() 
        cur.execute(procedures)
        cur.close()       

    def cursor_one_column(self, sql, arr=[]):
        """Returns un array with the results of the column"""
        cur=self._con.cursor()
        cur.execute(sql, arr)
        for row in cur:
            arr.append(row[0])
        cur.close()
        return arr

    def cursor_one_field(self, sql, arr=[]):
        """Returns only one field"""
        cur=self._con.cursor()
        cur.execute(sql, arr)
        row=cur.fetchone()[0]
        cur.close()
        return row

    def commit(self):
        self._con.commit()

    def rollback(self):
        self._con.rollback()

    def connection_string(self):
        return "dbname='{}' port='{}' user='{}' host='{}' password='{}'".format(self.db, self.port, self.user, self.server, self.password)

    ## Returns an url of the type psql://
    def url_string(self):
        return "psql://{}@{}:{}/{}".format(self.user, self.server, self.port, self.db)

    def connect(self, connection_string=None):
        """Used in code to connect using last self.strcon"""
        if connection_string==None:
            s=self.connection_string()
        else:
            s=connection_string
        try:
            self._con=DictConnection(s)
        except OperationalError as e:
            print('Unable to connect: {}'.format(e))
        self.init=datetime.datetime.now()

    def disconnect(self):
        self._con.close()

    ##Returns if connection is active
    def is_active(self):
        if self._con==None:
            return False
        return True

    def is_superuser(self):
        """Checks if the user has superuser role"""
        res=False
        cur=self.cursor()
        cur.execute("SELECT rolsuper FROM pg_roles where rolname=%s;", (self.user, ))
        if cur.rowcount==1:
            if cur.fetchone()[0]==True:
                res=True
        cur.close()
        return res
        
        
    ## Function to get password user PGPASSWORD environment or ask in console for it
    def get_password(self,  gettext_module=None, gettex_locale=None):
        try:
            import gettext
            t=gettext.translation(gettext_module,  gettex_locale)
            _=t.gettext
        except:
            _=str
        
        from os import environ
        from getpass import getpass
        try:
            self.password=environ['PGPASSWORD']
        except:
            print(_("Write the password for {}").format(self.url_string()))
            self.password=getpass()
        return self.password
        
## Function that adds an argparse argument group with connection parameters
## @param parser Argparse object
## @param gettext_module Gettext module
## @param gettex_locale Locale path
def argparse_connection_arguments_group(parser, gettext_module=None,  gettex_locale=None,  default_user="postgres", default_port=5432, default_server="127.0.0.1",  default_db="postgres"): 
    try:
        import gettext
        t=gettext.translation(gettext_module,  gettex_locale)
        _=t.gettext
    except:
        _=str

    group_db=parser.add_argument_group(_("Postgres database connection parameters"))
    group_db.add_argument('--user', help=_('Postgresql user'), default=default_user)
    group_db.add_argument('--port', help=_('Postgresql server port'), default=default_port)
    group_db.add_argument('--server', help=_('Postgresql server address'), default=default_server)
    group_db.add_argument('--db', help=_('Postgresql database'), default=default_db)
    
## Function that generate the start of a scritp just with connection arguments
def script_with_connection_arguments(name="",  description="", epilog="", version="", gettext_module=None, gettext_locale=None): 
    import argparse
    parser=argparse.ArgumentParser(prog=name, description=description, epilog=epilog, formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('--version', action='version', version=version)
    argparse_connection_arguments_group(parser, gettext_module, gettext_locale)

    global args
    args=parser.parse_args()

    con=Connection()

    con.user=args.user
    con.server=args.server
    con.port=args.port
    con.db=args.db
    
    con.get_password(gettext_module, gettext_locale)
    con.connect()
    return con


if __name__ == "__main__":
    con=script_with_connection_arguments("connection_pg_demo", "This is a connection script demo",  "Developed by Mariano Muñoz", "",  None, None)
    print("Is connection active?",  con.is_active())
    



from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QWidget
from xulpymoney.ui.Ui_wdgSimulationsAdd import Ui_wdgSimulationsAdd
from xulpymoney.libxulpymoney import Assets,  Simulation, DBAdmin
from xulpymoney.connection_pg_qt import ConnectionQt
from xulpymoney.libxulpymoneyfunctions import qmessagebox

class wdgSimulationsAdd(QWidget, Ui_wdgSimulationsAdd):
    def __init__(self, mem,   parent = None, name = None):
        """Simulations is the SimulationManager where the new simulation is going to be appended"""
        QWidget.__init__(self,  parent)
        self.setupUi(self)
        self.mem=mem
        self.parent=parent
        self.simulation=None#Simulation to be created
        self.mem.simulationtypes.qcombobox(self.cmbSimulationTypes)
        self.wdgStarting.set(self.mem, Assets(self.mem).first_datetime_with_user_data(), self.mem.localzone)
        self.wdgEnding.set(self.mem)
        self.simcon=None#Simulation connection

    @pyqtSlot()
    def on_buttonbox_accepted(self):
        if self.mem.con.is_superuser()==False:
            qmessagebox("The role of the user is not an administrator")
            self.parent.reject()
            return
        
        type_id=self.cmbSimulationTypes.itemData(self.cmbSimulationTypes.currentIndex())
        self.simulation=Simulation(self.mem, self.mem.con.db).init__create(type_id, self.wdgStarting.datetime(), self.wdgEnding.datetime())
        self.simulation.save()
        
        
        #Necesita solo roll superusuario y self.mem.con lo tiene
        createadmin=DBAdmin(self.mem.con)
        createadmin.create_db(self.simulation.simulated_db())
        self.mem.con.commit()

        #Crea nueva conexión
        self.con_sim=ConnectionQt().init__create(self.mem.con.user, self.mem.con.password, self.mem.con.server, self.mem.con.port, self.simulation.simulated_db())
        self.con_sim.connect()        
        admin=DBAdmin(self.con_sim)
        admin.xulpymoney_basic_schema()
        self.con_sim.commit()
        if self.simulation.type.id==1:#Copy between dates                    
            already_banks=self.con_sim.cursor_one_column("select id_entidadesbancarias from entidadesbancarias order by id_entidadesbancarias")
            mog=self.mem.con.mogrify("select * from entidadesbancarias where id_entidadesbancarias not in %s  order by id_entidadesbancarias", (tuple(already_banks), ))
            admin.copy(self.mem.con, mog,  "entidadesbancarias")
            
            admin.copy(self.mem.con, "select * from cuentas  order by id_cuentas",  "cuentas")
            
            admin.copy(self.mem.con, "select * from tarjetas  order by id_tarjetas",  "tarjetas")

            already_products=self.con_sim.cursor_one_column("select id from products order by id")#Rest personal products
            mog=self.mem.con.mogrify("select * from products where id not in %s  order by id", (tuple(already_products), ))
            admin.copy(self.mem.con, mog,  "products")
            
            admin.copy(self.mem.con, "select * from quotes ",  "quotes")
            
            admin.copy(self.mem.con, "select * from dps ",  "dps")
            
            admin.copy(self.mem.con, "select * from estimations_dps",  "estimations_dps")
            
            admin.copy(self.mem.con, "select * from estimations_eps ",  "estimations_eps")
            
            admin.copy(self.mem.con, "select * from inversiones order by id_inversiones",  "inversiones")
            
            admin.copy(self.mem.con, "select * from dividends order by id_dividends",  "dividends")
            
            already_conceptos=self.con_sim.cursor_one_column("select id_conceptos from conceptos order by id_conceptos")
            mog=self.mem.con.mogrify("select * from conceptos where id_conceptos not in %s  order by id_conceptos ", (tuple(already_conceptos), ))
            admin.copy(self.mem.con, mog,  "conceptos")
            
        self.con_sim.commit()
        self.con_sim.disconnect()
        self.parent.accept()    
        
    @pyqtSlot()
    def on_buttonbox_rejected(self):
        self.parent.reject()



import os, sys, glob, datetime, psycopg2, psycopg2.extras, shutil
import pkg_resources
import argparse
import datetime
import getpass
import gettext
from xulpymoney.version import __version__, __versiondate__
from xulpymoney.connection_pg import Connection
from xulpymoney.libmanagers import ObjectManager
from xulpymoney.libxulpymoney import Product, Zone, StockMarket
from officegenerator import ODS_Write, Coord


try:
    t=gettext.translation('xulpymoney',pkg_resources.resource_filename("xulpymoney","locale"))
    _=t.gettext
except:
    _=str


class Mem:
    def __init__(self):
        self.con=Connection()

class DV:
    def __init__(self):
        self.date=None
        self.value=None

    def __repr__(self):
        return "DV {} = {}".format(self.date,self.value)

class SmmManager(ObjectManager):
    def __init__(self,ohcldailymanager,periods):
        ObjectManager.__init__(self)
        self.periods=periods
        for i in range(self.periods-1, ohcldailymanager.length()):
            smm=DV()
            smm.date=ohcldailymanager.arr[i].date
            smm.value=0
            for p in range(self.periods):
                smm.value=smm.value+ohcldailymanager.arr[i-p].close
            smm.value=smm.value/self.periods
            self.append(smm)

    def find_by_date(self,date):
        for o in self.arr:
            if o.date==date:
                return o
        return None

class SmmDiffManager(ObjectManager):
    def __init__(self,smmm):
        ObjectManager.__init__(self)
        for i in range(1,smmm.length()):
            diff=DV()
            diff.date=smmm.arr[i].date
            diff.value=smmm.arr[i].value-smmm.arr[i-1].value
            self.append(diff)
    def find_by_date(self,date):
        for o in self.arr:
            if o.date==date:
                return o
        return None

## Almacena diff values
class Racha(ObjectManager):
    def __init__(self,ohcldailymanager):
        ObjectManager.__init__(self)
        self.ohcldailymanager=ohcldailymanager

    def __repr__(self):
        if self.is_positive():
            return "Racha positiva {}:{} con {} diffs".format(self.arr[0].date, self.arr[self.length()-1].date, self.length())
        else:
            return "Racha negativa {}:{} con {} diffs".format(self.arr[0].date, self.arr[self.length()-1].date, self.length())

#    ## CREO QUE MAL
#    def diff(self):
#        return self.ohcldailymanager.find(self.arr[self.length()-1].date).close-self.ohcldailymanager.find(self.arr[0].date).close

    def is_positive(self):
        if self.arr[0].value>=0:
            return True
        return False


## Almacena diff values
class RachaManager(ObjectManager):
    def __init__(self,ohcldailymanager):
        ObjectManager.__init__(self)
        self.ohcldailymanager=ohcldailymanager
   
    def suma_contenido_rachas(self):
        r=0
        for racha in self.arr:
            r=r+racha.length()
        return r

    ## Diferencia del ohcl, es decir del producto
    def diffProduct(self,racha):
        ind=self.arr.index(racha)
        if ind==self.length()-1:
            primera_fecha_racha_actual=racha.arr[0].date
            primera_fecha_racha_siguiente=racha.arr[racha.length()-1].date
        else:
            racha_siguiente=self.arr[ind+1]
            primera_fecha_racha_actual=racha.arr[0].date
            primera_fecha_racha_siguiente=racha_siguiente.arr[0].date
        ohcl_primera_fecha_racha_actual=self.ohcldailymanager.find(primera_fecha_racha_actual).close
        ohcl_primera_fecha_racha_siguiente=self.ohcldailymanager.find(primera_fecha_racha_siguiente).close
        return ohcl_primera_fecha_racha_siguiente-ohcl_primera_fecha_racha_actual

    ## Se resta el primera de la siguiente racha que es el precio en el que ha cambiado el signo del cdf, con el primer precio de la racha, que fue cando cambie el signo del cdf
    ## Se evalua si estoy a cortos o a largos
    def diffRacha(self,racha):
        value= self.diffProduct(racha)
        if racha.is_positive()==True and value>=0:
            return value
        elif racha.is_positive()==True and value<0:
            return value
        elif racha.is_positive()==False and value>=0:
            return -value
        elif racha.is_positive()==False and value<0:
            return -value

    def suma_diffs_year(self,year):
        r=0
        for racha in self.arr:
            if racha.arr[0].date.year==year:
                r=r+self.diffRacha(racha)
        return r


### MAIN SCRIPT ###
def main(parameters=None):
    parser=argparse.ArgumentParser(prog='xulpymoney', description=_('Generate your personal movie collection book'), epilog=_("Developed by Mariano Muñoz 2012-{}".format(__versiondate__.year)), formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('--version', action='version', version=__version__)

    group = parser.add_argument_group("productrequired=True")
    group.add_argument('--id', help=_('Insert films from current numbered directory'), action="store", default=79329)
    group.add_argument('--periods', help=_('Insert films from current numbered directory'), action="store", default=5)

    group_db=parser.add_argument_group(_("Postgres database connection parameters"))
    group_db.add_argument('--user', help=_('Postgresql user'), default='postgres')
    group_db.add_argument('--port', help=_('Postgresql server port'), default=5432)
    group_db.add_argument('--server', help=_('Postgresql server address'), default='127.0.0.1')
    group_db.add_argument('--db', help=_('Postgresql database'), default='xulpymoney')

    mem=Mem()
    global args
    args=parser.parse_args(parameters)
    args.periods=int(args.periods)
    args.id=int(args.id)


    mem.con.user=args.user
    mem.con.server=args.server
    mem.con.port=args.port
    mem.con.db=args.db

    print(_("Write the password for {}").format(mem.con.url_string()))
    mem.con.password=getpass.getpass()
    mem.con.connect()

    strdate=str(datetime.date.today()).replace("-","")
    doc=ODS_Write("{} Xulpymoney Report. SMM{}.ods".format(strdate,args.periods))
    doc.setMetadata("Xulpymoney SMM Report",  "SMM report", "Xulpymoney-{}".format(__version__))
    s1=doc.createSheet("MM5")
    s1.add("A1",[["Fecha","Ibex35","MMS5","Diff","Diff racha", "Gains"]], "OrangeCenter")
    product=Product(mem)
    product.id=args.id
    #product.zone=Zone(mem,1,'Europe/Madrid', None)
    #product.stockmarket=StockMarket(mem).init__create( 1, "Bolsa de Madrid", "es", datetime.time(9, 0), datetime.time(17, 38), "Europe/Madrid")
    product.needStatus(2)

    smmm=SmmManager(product.result.ohclDaily, args.periods)
    smmdm=SmmDiffManager(smmm)

    racham=RachaManager(product.result.ohclDaily)
    racha=Racha(product.result.ohclDaily)
    racham.append(racha)
    for i, o in enumerate(smmdm.arr):
        s1.add(Coord("A2").addRow(i), o.date, "WhiteDate")
        s1.add(Coord("B2").addRow(i), product.result.ohclDaily.find(o.date).close, "WhiteDecimal2")
        s1.add(Coord("C2").addRow(i), smmm.find_by_date(o.date).value, "WhiteDecimal2")
        s1.add(Coord("D2").addRow(i), o.value, "WhiteDecimal2")

        if racha.length()==0:
            racha.append(o)
        elif o.value>=0 and racha.arr[0].value>=0:
            racha.append(o)
        elif o.value<0 and racha.arr[0].value<0:
            racha.append(o)
        elif o.value>=0 and racha.arr[0].value<0:
            racha=Racha(product.result.ohclDaily)
            racha.append(o)
            racham.append(racha)
        elif o.value<0 and racha.arr[0].value>=0:
            racha=Racha(product.result.ohclDaily)
            racha.append(o)
            racham.append(racha)
    s1.setCursorPosition(Coord("A1").addRow(smmdm.length()))
    s1.setSplitPosition(Coord("A2"))


    s2=doc.createSheet("Rachas")
    s2.add("A1",[["Desde","Hasta","Días","Diff Product", "Gains Product"]], "OrangeCenter")

    for i, racha in enumerate(racham.arr):
        s2.add(Coord("A2").addRow(i), racha.arr[0].date, "WhiteDate")
        s2.add(Coord("B2").addRow(i), racha.arr[racha.length()-1].date, "WhiteDate")
        s2.add(Coord("C2").addRow(i), (racha.arr[racha.length()-1].date-racha.arr[0].date).days, "WhiteInteger")
        s2.add(Coord("D2").addRow(i), racham.diffProduct(racha), "WhiteDecimal2")
        s2.add(Coord("E2").addRow(i), racham.diffRacha(racha), "WhiteDecimal2")
        sentido="LARGOS" if racha.is_positive() else "CORTOS"
        s2.add(Coord("F2").addRow(i), sentido, "WhiteDecimal2")


    for year in range(datetime.date.today().year-10,datetime.date.today().year+1):
         print("{}: {}".format(year,racham.suma_diffs_year(year)))

    print ("Hay {} ohcls".format(product.result.ohclDaily.length()))
    print ("Hay {} diffs".format(smmdm.length()))
    print ("Hay {} rachas".format(racham.length()))
    print ("Hay {} diffs en rachas".format(racham.suma_contenido_rachas()))
    mem.con.disconnect()

    doc.save()


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgProductsDataMove.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgProductsDataMove(object):
    def setupUi(self, wdgProductsDataMove):
        wdgProductsDataMove.setObjectName("wdgProductsDataMove")
        wdgProductsDataMove.setWindowModality(QtCore.Qt.WindowModal)
        wdgProductsDataMove.resize(1108, 358)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(wdgProductsDataMove)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(wdgProductsDataMove)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.label = QtWidgets.QLabel(wdgProductsDataMove)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.table = myQTableWidget(wdgProductsDataMove)
        self.table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.table.setObjectName("table")
        self.table.setColumnCount(10)
        self.table.setRowCount(2)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(8, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(9, item)
        item = QtWidgets.QTableWidgetItem()
        brush = QtGui.QBrush(QtGui.QColor(255, 192, 192))
        brush.setStyle(QtCore.Qt.NoBrush)
        item.setBackground(brush)
        self.table.setItem(1, 0, item)
        self.horizontalLayout.addWidget(self.table)
        self.cmdInterchange = QtWidgets.QToolButton(wdgProductsDataMove)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/transfer.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdInterchange.setIcon(icon)
        self.cmdInterchange.setObjectName("cmdInterchange")
        self.horizontalLayout.addWidget(self.cmdInterchange)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.chkInvestments = QtWidgets.QCheckBox(wdgProductsDataMove)
        self.chkInvestments.setObjectName("chkInvestments")
        self.verticalLayout.addWidget(self.chkInvestments)
        self.cmd = QtWidgets.QPushButton(wdgProductsDataMove)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/merge.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmd.setIcon(icon1)
        self.cmd.setObjectName("cmd")
        self.verticalLayout.addWidget(self.cmd)
        self.horizontalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(wdgProductsDataMove)
        QtCore.QMetaObject.connectSlotsByName(wdgProductsDataMove)

    def retranslateUi(self, wdgProductsDataMove):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgProductsDataMove", "Move data between products"))
        self.label.setText(_translate("wdgProductsDataMove", "All product data will be moved from origin product to destiny one. If there are invesments created with origin product, they will be referenced to destiny product.\n"
"If origin is a user product, you can delete afterwards, because it won\'t have any data"))
        item = self.table.verticalHeaderItem(0)
        item.setText(_translate("wdgProductsDataMove", "Origin"))
        item = self.table.verticalHeaderItem(1)
        item.setText(_translate("wdgProductsDataMove", "Destiny"))
        item = self.table.horizontalHeaderItem(0)
        item.setText(_translate("wdgProductsDataMove", "Id"))
        item = self.table.horizontalHeaderItem(1)
        item.setText(_translate("wdgProductsDataMove", "Name"))
        item = self.table.horizontalHeaderItem(2)
        item.setText(_translate("wdgProductsDataMove", "ISIN"))
        item = self.table.horizontalHeaderItem(3)
        item.setText(_translate("wdgProductsDataMove", "Quotes"))
        item = self.table.horizontalHeaderItem(4)
        item.setText(_translate("wdgProductsDataMove", "DPS"))
        item = self.table.horizontalHeaderItem(5)
        item.setText(_translate("wdgProductsDataMove", "Investments"))
        item = self.table.horizontalHeaderItem(6)
        item.setText(_translate("wdgProductsDataMove", "Oportunities"))
        item = self.table.horizontalHeaderItem(7)
        item.setText(_translate("wdgProductsDataMove", "Splits"))
        item = self.table.horizontalHeaderItem(8)
        item.setText(_translate("wdgProductsDataMove", "DPS estimations"))
        item = self.table.horizontalHeaderItem(9)
        item.setText(_translate("wdgProductsDataMove", "EPS Estimations"))
        __sortingEnabled = self.table.isSortingEnabled()
        self.table.setSortingEnabled(False)
        self.table.setSortingEnabled(__sortingEnabled)
        self.cmdInterchange.setText(_translate("wdgProductsDataMove", "..."))
        self.chkInvestments.setText(_translate("wdgProductsDataMove", "Change personal investments with origin product to destiny product"))
        self.cmd.setText(_translate("wdgProductsDataMove", "Move data"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc



import datetime
from xulpymoney.libxulpymoney import EstimationDPS, EstimationEPS
from PyQt5.QtWidgets import QDialog
from xulpymoney.ui.Ui_frmEstimationsAdd import Ui_frmEstimationsAdd

class frmEstimationsAdd(QDialog, Ui_frmEstimationsAdd):
    def __init__(self, mem,  product,  type, parent=None):
        """type="dps or "eps" """
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.type=type
        self.product=product
        self.lbl.setText(self.product.name)
        self.txtYear.setText(str(datetime.date.today().year))
        self.txtDPA.selectAll()
        if self.type=="dps":
            self.setWindowTitle(self.tr("New dividend per share estimation"))
            self.lblEstimation.setText(self.tr("Add a dividend per share estimation"))
            self.cmd.setText(self.tr("Save a dividend per share estimation"))
        else:
            self.setWindowTitle(self.tr("New earnings per share estimation"))
            self.lblEstimation.setText(self.tr("Add a earning per share estimation"))
            self.cmd.setText(self.tr("Save a earning per share estimation"))

    def on_cmd_released(self):
        if self.type=="dps":
            d=EstimationDPS(self.mem).init__from_db(self.product, int(self.txtYear.text()) )##Lo carga si existe de la base de datos
        else:
            d=EstimationEPS(self.mem).init__from_db(self.product, int(self.txtYear.text()) )##Lo carga si existe de la base de datos

        d.estimation=self.txtDPA.decimal()
        d.manual=True
        d.source="Internet"
        d.date_estimation=datetime.date.today()
        d.save()
        self.mem.con.commit()      
        if self.type=="dps":
            self.product.estimations_dps.load_from_db()
        else:
            self.product.estimations_eps.load_from_db()
            
        self.accept()



from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import  QWidget, QProgressDialog
from xulpymoney.ui.Ui_wdgAPR import Ui_wdgAPR
from xulpymoney.libxulpymoney import Assets, Money, Percentage
from xulpymoney.libxulpymoneyfunctions import qcenter, none2decimal0,  qright
from xulpymoney.libxulpymoneytypes import eConcept

from decimal import Decimal
import datetime

class wdgAPR(QWidget, Ui_wdgAPR):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.tab.setCurrentIndex(0)
        self.mem=mem
        self.progress = QProgressDialog(self.tr("Filling data of the report"), self.tr("Cancel"), 0,0)
        self.progress.setModal(True)
        self.progress.setWindowTitle(self.tr("Calculating data..."))
        self.progress.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
        self.progress.setMinimumDuration(0)        
        self.table.settings(self.mem, "wdgAPR")
        self.tblReport.settings(self.mem, "wdgAPR")
         
        
        firstyear=Assets(self.mem).first_datetime_with_user_data().year        
        currentyear=int(self.mem.settingsdb.value("wdgAPR/cmbYear", firstyear))
        self.wdgYear.initiate(firstyear,  datetime.date.today().year, currentyear)#Push an wdgYear changed
        self.wdgYear.changed.connect(self.on_my_wdgYear_changed)
        self.on_my_wdgYear_changed()
        

    def on_my_wdgYear_changed(self):
        print ("on_wdgYear_changed", self.wdgYear.year)
        self.mem.settingsdb.setValue("wdgAPR/cmbYear", self.wdgYear.year )
        self.progress.reset()
        self.progress.setMinimum(0)
        self.progress.setMaximum((datetime.date.today().year-self.wdgYear.year+1)*2)
        self.progress.forceShow()
        self.progress.setValue(0)
        
        self.table.clear()
        self.tblReport.clear()
        self.dates=[]
        self.incomes=[]
        self.expenses=[]
        self.gains=[]
        self.dividends=[]

        self.load_data()
        self.load_report()


    def load_data(self):        
        inicio=datetime.datetime.now()       
        anoinicio=self.wdgYear.year
        anofinal=datetime.date.today().year
        
        self.table.applySettings()
        self.table.setRowCount(anofinal-anoinicio+1+1)
        lastsaldo=Money(self.mem)
        sumdividends=Money(self.mem)
        sumgains=Money(self.mem)
        sumexpenses=Money(self.mem)
        sumincomes=Money(self.mem)
        sumicdg=Money(self.mem)
        for i in range(anoinicio, anofinal+1):
            if self.progress.wasCanceled():
                break;
            else:
                self.progress.setValue(self.progress.value()+1)                     
            si=lastsaldo
            sf=Assets(self.mem).saldo_total(self.mem.data.investments,  datetime.date(i, 12, 31))
            expenses=Assets(self.mem).saldo_anual_por_tipo_operacion( i,1)#+Assets(self.mem).saldo_anual_por_tipo_operacion (cur,i, 7)#expenses + Facturación de tarjeta
            dividends=Assets(self.mem).dividends_neto( i)
            incomes=Assets(self.mem).saldo_anual_por_tipo_operacion(  i,2)-dividends #Se quitan los dividends que luego se suman
            gains=Assets(self.mem).consolidado_neto(self.mem.data.investments,  i)
            
            self.dates.append(datetime.datetime(i, 12, 31))
            self.expenses.append(-expenses.amount)
            self.dividends.append(dividends.amount)
            self.incomes.append(incomes.amount)
            self.gains.append(gains.amount)

            gi=incomes+dividends+gains+expenses     
            self.table.setItem(i-anoinicio, 0, qcenter(str(i)))
            self.table.setItem(i-anoinicio, 1, si.qtablewidgetitem())
            self.table.setItem(i-anoinicio, 2, sf.qtablewidgetitem())
            self.table.setItem(i-anoinicio, 3, (sf-si).qtablewidgetitem())
            self.table.setItem(i-anoinicio, 4, incomes.qtablewidgetitem())
            self.table.setItem(i-anoinicio, 5, gains.qtablewidgetitem())
            self.table.setItem(i-anoinicio, 6, dividends.qtablewidgetitem())
            self.table.setItem(i-anoinicio, 7, expenses.qtablewidgetitem())
            self.table.setItem(i-anoinicio, 8, gi.qtablewidgetitem())
            sumdividends=sumdividends+dividends
            sumgains=sumgains+gains
            sumexpenses=sumexpenses+expenses
            sumincomes=sumincomes+incomes
            sumicdg=sumicdg+gi
            self.table.setItem(i-anoinicio, 9, Percentage(sf -si, si).qtablewidgetitem())
            lastsaldo=sf
        self.table.setItem(anofinal-anoinicio+1, 0, qcenter((self.tr("TOTAL"))))
        self.table.setItem(anofinal-anoinicio+1, 4, sumincomes.qtablewidgetitem())
        self.table.setItem(anofinal-anoinicio+1, 5, sumgains.qtablewidgetitem())
        self.table.setItem(anofinal-anoinicio+1, 6, sumdividends.qtablewidgetitem())
        self.table.setItem(anofinal-anoinicio+1, 7, sumexpenses.qtablewidgetitem())
        self.table.setItem(anofinal-anoinicio+1, 8, sumicdg.qtablewidgetitem())
        final=datetime.datetime.now()          
        print ("wdgAPR > load_data: {}".format(final-inicio))

    def load_report(self):        
        print ("Initializating load_report")
        inicio=datetime.datetime.now()       
        anoinicio=self.wdgYear.year
        anofinal=datetime.date.today().year
        sumgd=Money(self.mem, 0, self.mem.localcurrency)
        sumtaxes=Decimal(0)
        sumcomissions=Decimal(0)
        self.tblReport.applySettings()
        self.tblReport.setRowCount(anofinal-anoinicio+1+1)
        for i in range(anoinicio, anofinal+1):
            if self.progress.wasCanceled():
                break;
            else:
                self.progress.setValue(self.progress.value()+1)                     
            sinvested=Assets(self.mem).invested(datetime.date(i, 12, 31))
            sbalance=Assets(self.mem).saldo_todas_inversiones(self.mem.data.investments, datetime.date(i, 12, 31))
            gd=Assets(self.mem).consolidado_neto(self.mem.data.investments,  i)+Assets(self.mem).dividends_neto(i)
            sumgd=sumgd+gd

            self.tblReport.setItem(i-anoinicio, 0, qcenter(i))
            self.tblReport.setItem(i-anoinicio, 1, sinvested.qtablewidgetitem())
            self.tblReport.setItem(i-anoinicio, 2, sbalance.qtablewidgetitem())
            self.tblReport.setItem(i-anoinicio, 3, (sbalance-sinvested).qtablewidgetitem())
            self.tblReport.setItem(i-anoinicio, 4, Percentage(sbalance-sinvested, sinvested).qtablewidgetitem())
            self.tblReport.setItem(i-anoinicio, 6, gd.qtablewidgetitem())

            taxes=none2decimal0(self.mem.con.cursor_one_field("select sum(importe) from opercuentas where id_conceptos in (%s, %s) and date_part('year',datetime)=%s", (int(eConcept.TaxesReturn), int(eConcept.TaxesPayment), i)))
            self.tblReport.setItem(i-anoinicio, 8, qright(taxes))
            sumtaxes=sumtaxes+taxes

            comissions=none2decimal0(self.mem.con.cursor_one_field("""
select 
    sum(suma) 
from (
            select 
                sum(importe) as suma 
            from 
                opercuentas 
            where 
                id_conceptos in (%s, %s) and  
                date_part('year',datetime)=%s
            union 
            select 
                -sum(comision) as suma 
            from 
                operinversiones 
            where  
                date_part('year',datetime)=%s
        ) as uni""", (int(eConcept.BankCommissions), int(eConcept.CommissionCustody), i,i)))
            self.tblReport.setItem(i-anoinicio, 9, qright(comissions))
            sumcomissions=sumcomissions+comissions
            
            

        self.tblReport.setItem(anofinal-anoinicio+1, 0, qcenter((self.tr("TOTAL"))))
        self.tblReport.setItem(anofinal-anoinicio+1, 6, sumgd.qtablewidgetitem())
        self.tblReport.setItem(anofinal-anoinicio+1, 8, qright(sumtaxes))
        self.tblReport.setItem(anofinal-anoinicio+1, 9, qright(sumcomissions))
        
        lastyear=datetime.date(datetime.date.today().year, 12, 31)
        diff=Assets(self.mem).saldo_todas_inversiones(self.mem.data.investments, lastyear)-Assets(self.mem).invested(lastyear)
        s=""
        s=self.tr("From {} I have generated {}.").format(self.wdgYear.year, sumgd)
        s=s+"\n"+self.tr("Difference between invested amount and current invesment balance is {}").format(diff)
        if (diff+sumgd).isGETZero():
            s=s+"\n"+self.tr("So I'm wining {} which is {} per year.").format(sumgd+diff, self.mem.localcurrency.string((sumgd+diff).amount/(datetime.date.today().year-self.wdgYear.year+1)))
        else:
            s=s+"\n"+self.tr("So I'm losing {} which is {} per year.").format(sumgd+diff, self.mem.localcurrency.string((sumgd+diff).amount/(datetime.date.today().year-self.wdgYear.year+1)))
        
        self.lblReport.setText(s)
        
        
        final=datetime.datetime.now()          
        print ("wdgAPR > load_report: {0}".format(final-inicio))



## @namespace xulpymoney.libxulpymoneyfunctions
## @brief Package with all xulpymoney auxiliar functions.
from PyQt5.QtCore import Qt,  QCoreApplication, QLocale
from PyQt5.QtGui import QIcon,  QColor
from PyQt5.QtWidgets import QTableWidgetItem,  QWidget,  QMessageBox, QApplication, QCheckBox, QHBoxLayout
from decimal import Decimal
from os import path, makedirs
import datetime
import time
import functools
import warnings
import inspect
import logging
import pytz
import sys
from xulpymoney.version import __version__, __versiondate__
from xulpymoney.libxulpymoneytypes import eDtStrings

## Sets debug sustem, needs
## @param args It's the result of a argparse     args=parser.parse_args()        
def addDebugSystem(args):
    logFormat = "%(asctime)s.%(msecs)03d %(levelname)s %(message)s [%(module)s:%(lineno)d]"
    dateFormat='%F %I:%M:%S'

    if args.debug=="DEBUG":#Show detailed information that can help with program diagnosis and troubleshooting. CODE MARKS
        logging.basicConfig(level=logging.DEBUG, format=logFormat, datefmt=dateFormat)
    elif args.debug=="INFO":#Everything is running as expected without any problem. TIME BENCHMARCKS
        logging.basicConfig(level=logging.INFO, format=logFormat, datefmt=dateFormat)
    elif args.debug=="WARNING":#The program continues running, but something unexpected happened, which may lead to some problem down the road. THINGS TO DO
        logging.basicConfig(level=logging.WARNING, format=logFormat, datefmt=dateFormat)
    elif args.debug=="ERROR":#The program fails to perform a certain function due to a bug.  SOMETHING BAD LOGIC
        logging.basicConfig(level=logging.ERROR, format=logFormat, datefmt=dateFormat)
    elif args.debug=="CRITICAL":#The program encounters a serious error and may stop running. ERRORS
        logging.basicConfig(level=logging.CRITICAL, format=logFormat, datefmt=dateFormat)

    logging.info("Debug level set to {}".format(args.debug))
    
## Adds the commons parameter of the program to argparse
## @param parser It's a argparse.ArgumentParser
def addCommonToArgParse(parser):
    parser.add_argument('--version', action='version', version="{} ({})".format(__version__, __versiondate__))
    parser.add_argument('--debug', help="Debug program information", choices=["DEBUG","INFO","WARNING","ERROR","CRITICAL"], default="ERROR")

## Function to conver hour strings with AM/PM to a iso string time
## @param s Is a string for time with AMPM and returns a 24 hours time string with zfill. AM/PM can be upper or lower case
## @param type Integer that can have this options 1: "5:35PM"
## @return String with hour in iso mode: "17:35"
def ampm2stringtime(s, type):
    s=s.upper()
    if type==1:#5:35PM > 17:35   ó 5:35AM > 05:35
        s=s.replace("AM", "")
        if s.find("PM"):
            s=s.replace("PM", "")
            points=s.split(":")
            s=str(int(points[0])+12).zfill(2)+":"+points[1]
        else:#AM
            points=s.split(":")
            s=str(int(points[0])).zfill(2)+":"+points[1]
        return s

## Changes zoneinfo from a dtaware object
## For example:
## - datetime.datetime(2018, 5, 18, 8, 12, tzinfo=<DstTzInfo 'Europe/Madrid' CEST+2:00:00 DST>)
## - libxulpymoneyfunctions.dtaware_changes_tz(a,"Europe/London")
## - datetime.datetime(2018, 5, 18, 7, 12, tzinfo=<DstTzInfo 'Europe/London' BST+1:00:00 DST>)
## @param dt datetime aware object
## @tzname String with datetime zone. For example: "Europe/Madrid"
## @return datetime aware object
def dtaware_changes_tz(dt,  tzname):
    if dt==None:
        return None
    tzt=pytz.timezone(tzname)
    tarjet=tzt.normalize(dt.astimezone(tzt))
    return tarjet

## Creates a QTableWidgetItem with the date
def qdate(date):
    if date==None:
        return qempty()
    return qcenter(str(date))

def qmessagebox(text):
    m=QMessageBox()
    m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
    m.setIcon(QMessageBox.Information)
    m.setText(text)
    m.exec_()   

def sync_data(con_source, con_target, progress=None):
    """con is con_target, 
    progress is a pointer to progressbar
    returns a tuple (numberofproductssynced, numberofquotessynced)"""
    #Checks if database has same version
    cur_target=con_target.cursor()
    cur2_target=con_target.cursor()
    cur_source=con_source.cursor()
    
    
    #Checks if database has same version
    cur_source.execute("select value from globals where id_globals=1")
    cur_target.execute("select value from globals where id_globals=1")
    
    if cur_source.fetchone()[0]!=cur_target.fetchone()[0]:
        logging.critical ("Databases has diferent versions, please update them")
        sys.exit(0)
    
    quotes=0#Number of quotes synced
    estimation_dps=0#Number of estimation_dps synced
    estimation_eps=0#Number of estimation_eps synced
    dps=0
    splits=0 #Number of splits synced
    products=0#Number of products synced
    
    #Iterate all products
    cur_target.execute("select id,name from products where id>0 order by name;")
    logging.info ("Syncing {} products".format (cur_target.rowcount))
    for row in cur_target:
        output="{}: ".format(row['name'])
        ## QUOTES #####################################################################
        #Search last datetime
        cur2_target.execute("select max(datetime) as max from quotes where id=%s", (row['id'], ))
        max=cur2_target.fetchone()[0]
        #Ask for quotes in source with last datetime
        if max==None:#No hay ningun registro y selecciona todos
            cur_source.execute("select * from quotes where id=%s", (row['id'], ))
        else:#Hay registro y selecciona los posteriores a el
            cur_source.execute("select * from quotes where id=%s and datetime>%s", (row['id'], max))
        if cur_source.rowcount!=0:
            print("  - Syncing {} since {} ".format(row['name'], max),end="")
            for  row_source in cur_source: #Inserts them 
                cur2_target.execute("insert into quotes (id, datetime, quote) values (%s,%s,%s)", ( row_source['id'], row_source['datetime'], row_source['quote']))
                quotes=quotes+1
                output=output+"."
                
        ## DPS ################################################################################
        #Search last datetime
        cur2_target.execute("select max(date) as max from dps where id=%s", (row['id'], ))
        max=cur2_target.fetchone()[0]
        #Ask for quotes in source with last datetime
        if max==None:#No hay ningun registro y selecciona todos
            cur_source.execute("select * from dps where id=%s", (row['id'], ))
        else:#Hay registro y selecciona los posteriores a el
            cur_source.execute("select * from dps where id=%s and date>%s", (row['id'], max))
        if cur_source.rowcount!=0:
            for  row_source in cur_source: #Inserts them 
                cur2_target.execute("insert into dps (date, gross, id) values (%s,%s,%s)", ( row_source['date'], row_source['gross'], row_source['id']))
                dps=dps+1
                output=output+"-"

        ## DPS ESTIMATIONS #####################################################################
        #Search last datetime
        cur2_target.execute("select max(year) as max from estimations_dps where id=%s", (row['id'], ))
        max=cur2_target.fetchone()[0]
        #Ask for quotes in source with last datetime
        if max==None:#No hay ningun registro y selecciona todos
            cur_source.execute("select * from estimations_dps where id=%s", (row['id'], ))
        else:#Hay registro y selecciona los posteriores a el
            cur_source.execute("select * from estimations_dps where id=%s and year>%s", (row['id'], max))
        if cur_source.rowcount!=0:
            for  row_source in cur_source: #Inserts them 
                cur2_target.execute("insert into estimations_dps (year, estimation, date_estimation, source, manual, id) values (%s,%s,%s,%s,%s,%s)", ( row_source['year'], row_source['estimation'], row_source['date_estimation'], row_source['source'], row_source['manual'],  row_source['id']))
                estimation_dps=estimation_dps+1
                output=output+"+"
                
        ## EPS ESTIMATIONS #####################################################################
        #Search last datetime
        cur2_target.execute("select max(year) as max from estimations_eps where id=%s", (row['id'], ))
        max=cur2_target.fetchone()[0]
        #Ask for quotes in source with last datetime
        if max==None:#No hay ningun registro y selecciona todos
            cur_source.execute("select * from estimations_eps where id=%s", (row['id'], ))
        else:#Hay registro y selecciona los posteriores a el
            cur_source.execute("select * from estimations_eps where id=%s and year>%s", (row['id'], max))
        if cur_source.rowcount!=0:
            for  row_source in cur_source: #Inserts them 
                cur2_target.execute("insert into estimations_eps (year, estimation, date_estimation, source, manual, id) values (%s,%s,%s,%s,%s,%s)", ( row_source['year'], row_source['estimation'], row_source['date_estimation'], row_source['source'], row_source['manual'],  row_source['id']))
                estimation_eps=estimation_eps+1
                output=output+"*"
                
        ## SPLITS  #####################################################################
        #Search last datetime
        cur2_target.execute("select max(datetime) as max from splits where products_id=%s", (row['id'], ))
        max=cur2_target.fetchone()[0]
        #Ask for quotes in source with last datetime
        if max==None:#No hay ningun registro y selecciona todos
            cur_source.execute("select * from splits where products_id=%s", (row['id'], ))
        else:#Hay registro y selecciona los posteriores a el
            cur_source.execute("select * from splits where products_id=%s and datetime>%s", (row['id'], max))
        if cur_source.rowcount!=0:
            for  row_source in cur_source: #Inserts them 
                cur2_target.execute("insert into splits (datetime, products_id, before, after, comment) values (%s,%s,%s,%s,%s)", ( row_source['datetime'], row_source['products_id'], row_source['before'], row_source['after'], row_source['comment']))
                splits=splits+1
                output=output+"s"

        if output!="{}: ".format(row['name']):
            products=products+1
            logging.debug(output)
            
        if progress!=None:#If there's a progress bar
            progress.setValue(cur_target.rownumber)
            progress.setMaximum(cur_target.rowcount)
            QCoreApplication.processEvents()
    con_target.commit()
    
    if progress!=None:
        s=QCoreApplication.translate("Core", """From {} desynchronized products added:
    - {} quotes
    - {} dividends per share
    - {} dividend per share estimations
    - {} earnings per share estimations
    - {} splits / contrasplits""").format(  products,  quotes, dps, estimation_dps,  estimation_eps, splits)
            
        qmessagebox(s)  
    
def dtaware2epochms(d):
    """
        Puede ser dateime o date
        Si viene con zona datetime zone aware, se convierte a UTC y se da el valor en UTC
        return datetime.datetime.now(pytz.timezone(self.name))
    """
    if d.__class__==datetime.datetime:
        if d.tzname()==None:#unaware datetine
            logging.critical("Must be aware")
        else:#aware dateime changed to unawar
            utc=dtaware_changes_tz(d, 'UTC')
            return utc.timestamp()*1000
    logging.critical("{} can't be converted to epochms".format(d.__class__))
    
## Return a UTC datetime aware
def epochms2dtaware(n):
    utc_unaware=datetime.datetime.utcfromtimestamp(n/1000)
    utc_aware=utc_unaware.replace(tzinfo=pytz.timezone('UTC'))
    return utc_aware


## Returns a formated string of a dtaware string formatting with a zone name
## @param dt datetime aware object
## @return String
def dtaware2string(dt, type=eDtStrings.QTableWidgetItem):
    if dt==None:
        resultado="None"
    elif dt.tzname()==None:
        resultado="Naive date and time"
    elif type==eDtStrings.QTableWidgetItem:
        if dt.microsecond==4 :
            resultado="{}-{}-{}".format(dt.year, str(dt.month).zfill(2), str(dt.day).zfill(2))
        else:
            resultado="{}-{}-{} {}:{}:{}".format(dt.year, str(dt.month).zfill(2), str(dt.day).zfill(2), str(dt.hour).zfill(2), str(dt.minute).zfill(2),  str(dt.second).zfill(2))
    elif type==eDtStrings.Filename:
            resultado="{}{}{} {}{}".format(dt.year, str(dt.month).zfill(2), str(dt.day).zfill(2), str(dt.hour).zfill(2), str(dt.minute).zfill(2))
    return resultado
    
## allows you to measure the execution time of the method/function by just adding the @timeit decorator on the method.
## @timeit
def timeit(method):
    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()

        if 'log_time' in kw:
            name = kw.get('log_name', method.__name__.upper())
            kw['log_time'][name] = int((te - ts) * 1000)
        else:
            print ('%r  %2.2f ms' % (method.__name__, (te - ts) * 1000))
        return result

    return timed

def deprecated(func):
     """This is a decorator which can be used to mark functions
     as deprecated. It will result in a warning being emitted
     when the function is used."""
     @functools.wraps(func)
     def new_func(*args, **kwargs):
         warnings.simplefilter('always', DeprecationWarning)  # turn off filter
         warnings.warn("Call to deprecated function {}.".format(func.__name__),
                       category=DeprecationWarning,
                       stacklevel=2)
         warnings.simplefilter('default', DeprecationWarning)  # reset filter
         return func(*args, **kwargs)
     return new_func
 
## dt es un datetime con timezone, que se mostrara con la zone pasado como parametro
## Convierte un datetime a string, teniendo en cuenta los microsehgundos, para ello se convierte a datetime local
def qdatetime(dt, zone):
    newdt=dtaware_changes_tz(dt, zone.name)
    if newdt==None:
        return qempty()
    a=QTableWidgetItem(dtaware2string(newdt))
    a.setTextAlignment(Qt.AlignVCenter|Qt.AlignRight)
    return a

def qtime(dt):
    """
        Shows the time of a datetime
    """
    if dt==None:
        return qempty()
    if dt.microsecond==5:
        item=qleft(str(dt)[11:-13])
        item.setBackground(QColor(255, 255, 148))
    elif dt.microsecond==4:
        item=qleft(str(dt)[11:-13])
        item.setBackground(QColor(148, 148, 148))
    else:
        item=qleft(str(dt)[11:-6])
    return item
    
def list2string(lista):
        """Covierte lista a string"""
        if  len(lista)==0:
            return ""
        if str(lista[0].__class__) in ["<class 'int'>", "<class 'float'>"]:
            resultado=""
            for l in lista:
                resultado=resultado+ str(l) + ", "
            return resultado[:-2]
        elif str(lista[0].__class__) in ["<class 'str'>",]:
            resultado=""
            for l in lista:
                resultado=resultado+ "'" + str(l) + "', "
            return resultado[:-2]
            
def string2list_of_integers(s, separator=", "):
    """Convers a string of integer separated by comma, into a list of integer"""
    arr=[]
    if s!="":
        arrs=s.split(separator)
        for a in arrs:
            arr.append(int(a))
    return arr
    
def string2date(iso, type=1):
    """
        date string to date, with type formats
    """
    if type==1: #YYYY-MM-DD
        d=iso.split("-")
        return datetime.date(int(d[0]), int(d[1]),  int(d[2]))
    if type==2: #DD/MM/YYYY
        d=iso.split("/")
        return datetime.date(int(d[2]), int(d[1]),  int(d[0]))

## Function to generate a datetime (aware or naive) from a string
## @param s String
## @param type Integer
## @param zone Name of the zone. By default "Europe Madrid" only in type 3and 4
## @return Datetime
def string2datetime(s, type, zone="Europe/Madrid"):
    if type==1:#2017-11-20 23:00:00+00:00  ==> Aware
        s=s[:-3]+s[-2:]
        dat=datetime.datetime.strptime( s, "%Y-%m-%d %H:%M:%S%z" )
        return dat
    if type==2:#20/11/2017 23:00 ==> Naive
        dat=datetime.datetime.strptime( s, "%d/%m/%Y %H:%M" )
        return dat
    if type==3:#20/11/2017 23:00 ==> Aware, using zone parameter
        dat=datetime.datetime.strptime( s, "%d/%m/%Y %H:%M" )
        z=pytz.timezone(zone)
        return z.localize(dat)
    if type==4:#27 1 16:54 2017==> Aware, using zone parameter . 1 es el mes convertido con month2int
        dat=datetime.datetime.strptime( s, "%d %m %H:%M %Y")
        z=pytz.timezone(zone)
        return z.localize(dat)
    if type==5:#2017-11-20 23:00:00.000000+00:00  ==> Aware with microsecond
        s=s[:-3]+s[-2:]#quita el :
        arrPunto=s.split(".")
        s=arrPunto[0]+s[-5:]
        micro=int(arrPunto[1][:-5])
        dat=datetime.datetime.strptime( s, "%Y-%m-%d %H:%M:%S%z" )
        dat=dat+datetime.timedelta(microseconds=micro)
        return dat

## Converts a tring 12:23 to a datetime.time object
def string2time(s):
    a=s.split(":")
    return datetime.time(int(a[0]), int(a[1]))

## Bytes 2 string
def b2s(b, code='UTF-8'):
    return b.decode(code)
    
def s2b(s, code='UTF8'):
    """String 2 bytes"""
    if s==None:
        return "".encode(code)
    else:
        return s.encode(code)

def c2b(state):
    """QCheckstate to python bool"""
    if state==Qt.Checked:
        return True
    else:
        return False

def b2c(booleano):
    """Bool to QCheckstate"""
    if booleano==True:
        return Qt.Checked
    else:
        return Qt.Unchecked     

def day_end(dattime, zone):
    """Saca cuando acaba el dia de un dattime en una zona concreta"""
    return dtaware_changes_tz(dattime, zone.name).replace(hour=23, minute=59, second=59)
    
def day_start(dattime, zone):
    return dtaware_changes_tz(dattime, zone.name).replace(hour=0, minute=0, second=0)
    
def day_end_from_date(date, zone):
    """Saca cuando acaba el dia de un dattime en una zona concreta"""
    return dtaware(date, datetime.time(23, 59, 59), zone.name)
    
def day_start_from_date(date, zone):
    return dtaware(date, datetime.time(0, 0, 0), zone.name)
    
def month_start(year, month, zone):
    """datetime primero de un mes
    """
    return day_start_from_date(datetime.date(year, month, 1), zone)
    
def month2int(s):
    """
        Converts a month string to a int
    """
    if s in ["Jan", "Ene", "Enero", "January", "enero", "january"]:
        return 1
    if s in ["Feb", "Febrero", "February", "febrero", "february"]:
        return 2
    if s in ["Mar", "Marzo", "March", "marzo", "march"]:
        return 3
    if s in ["Apr", "Abr", "April", "Abril", "abril", "april"]:
        return 4
    if s in ["May", "Mayo", "mayo", "may"]:
        return 5
    if s in ["Jun", "June", "Junio", "junio", "june"]:
        return 6
    if s in ["Jul", "July", "Julio", "julio", "july"]:
        return 7
    if s in ["Aug", "Ago", "August", "Agosto", "agosto", "august"]:
        return 8
    if s in ["Sep", "Septiembre", "September", "septiembre", "september"]:
        return 9
    if s in ["Oct", "October", "Octubre", "octubre", "october"]:
        return 10
    if s in ["Nov", "Noviembre", "November", "noviembre", "november"]:
        return 11
    if s in ["Dic", "Dec", "Diciembre", "December", "diciembre", "december"]:
        return 12

def month_end(year, month, zone):
    """datetime último de un mes
    """
    return day_end_from_date(month_last_date(year, month), zone)
    
## Returns a date with the last day of a month
## @return datetime.date object
def month_last_date(year, month):
    if month == 12:
        return datetime.date(year, month, 31)
    return datetime.date(year, month+1, 1) - datetime.timedelta(days=1)

## Returns an aware datetime with the start of year
def year_start(year, zone):
    return day_start_from_date(datetime.date(year, 1, 1), zone)
    
## Returns an aware datetime with the last of year
def year_end(year, zone):
    return day_end_from_date(datetime.date(year, 12, 31), zone)
    
## Function that converts a number of days to a string showing years, months and days
## @param days Integer with the number of days
## @return String like " 0 years, 1 month and 3 days"
def days2string(days):
    years=days//365
    months=(days-years*365)//30
    days=int(days -years*365 -months*30)
    if years==1:
        stryears=QApplication.translate("Core", "year")
    else:
        stryears=QApplication.translate("Core", "years")
    if months==1:
        strmonths=QApplication.translate("Core", "month")
    else:
        strmonths=QApplication.translate("Core", "months")
    if days==1:
        strdays=QApplication.translate("Core", "day")
    else:
        strdays=QApplication.translate("Core", "days")
    return QApplication.translate("Core", "{} {}, {} {} and {} {}").format(years, stryears,  months,  strmonths, days,  strdays)


def dirs_create():
    """
        Returns xulpymoney_tmp_dir, ...
    """
    dir_tmp=path.expanduser("~/.xulpymoney/tmp/")
    try:
        makedirs(dir_tmp)
    except:
        pass
    return dir_tmp

## Function to create a datetime aware object
## @param date datetime.date object
## @param hour datetime.hour object
## @param zonename String with datetime zone name. For example "Europe/Madrid"
## @return datetime aware
def dtaware(date, hour, zonename):
    z=pytz.timezone(zonename)
    a=datetime.datetime(date.year,  date.month,  date.day,  hour.hour,  hour.minute,  hour.second, hour.microsecond)
    a=z.localize(a)
    return a
    
## Converts strings True or False to boolean
## @param s String
## @return Boolean
def str2bool(s):
    if s=="True":
        return True
    return False    
## Converts boolean to  True or False string
## @param s String
## @return Boolean
def bool2string(b):
    if b==True:
        return "VERDADERO"
    return "FALSO"
    
## Function that converts a None value into a Decimal('0')
## @param dec Should be a Decimal value or None
## @return Decimal
def none2decimal0(dec):
    if dec==None:
        return Decimal('0')
    return dec

## Relation between gains and risk. Should be over 2 to be a good investment
## @param target Decimal with investment price target
## @param entry Decimal with investment entry price
## @param stoploss Decimal with investment stoploss price
## @return Decimal or None if relation can't be calculated
def relation_gains_risk(target, entry,  stoploss):
    try:
        return Decimal(abs(target-entry)/abs(entry-stoploss))
    except:
        return None


def qbool(bool):
    """Prints bool and check. Is read only and enabled"""
    if bool==None:
        return qempty()
    a=QTableWidgetItem()
    a.setFlags( Qt.ItemIsSelectable |  Qt.ItemIsEnabled )#Set no editable
    if bool:
        a.setCheckState(Qt.Checked);
        a.setText(QApplication.translate("Core","True"))
    else:
        a.setCheckState(Qt.Unchecked);
        a.setText(QApplication.translate("Core","False"))
    a.setTextAlignment(Qt.AlignVCenter|Qt.AlignCenter)
    return a
    
def wdgBool(bool):
    """Center checkbox
    Yo must use with table.setCellWidget(0,0,wdgBool)
    Is disabled to be readonly"""
    pWidget = QWidget()
    pCheckBox = QCheckBox();
    if bool:
        pCheckBox.setCheckState(Qt.Checked);
    else:
        pCheckBox.setCheckState(Qt.Unchecked);
    pLayout = QHBoxLayout(pWidget);
    pLayout.addWidget(pCheckBox);
    pLayout.setAlignment(Qt.AlignCenter);
    pLayout.setContentsMargins(0,0,0,0);
    pWidget.setLayout(pLayout);
    pCheckBox.setEnabled(False)
    return pWidget
    
## Returns a QTableWidgetItem representing an empty value
def qempty():
    a=QTableWidgetItem("---")
    a.setTextAlignment(Qt.AlignVCenter|Qt.AlignRight)
    return a

def qcenter(string, digits=None):
    if string==None:
        return qempty()
    a=QTableWidgetItem(str(string))
    a.setTextAlignment(Qt.AlignVCenter|Qt.AlignCenter)
    return a
    
def qleft(string):
    if string==None:
        return qempty()
    a=QTableWidgetItem(str(string))
    a.setTextAlignment(Qt.AlignVCenter|Qt.AlignLeft)
    return a

def qright(string, digits=None):
    """When digits, limits the number to """
    if string==None:
        return qempty()
    if string!=None and digits!=None:
        string=round(string, digits)
    a=QTableWidgetItem(str(string))
    a.setTextAlignment(Qt.AlignVCenter|Qt.AlignRight)
    try:#If is a number corized it
        if string==None:
            a.setForeground(QColor(0, 0, 255))
        elif string<0:
            a.setForeground(QColor(255, 0, 0))
    except:
        pass
    return a
        
def web2utf8(cadena):
    cadena=cadena.replace('&#209;','Ñ')
    cadena=cadena.replace('&#241;','ñ')
    cadena=cadena.replace('&#252;','ü')
    cadena=cadena.replace('&#246;','ö')
    cadena=cadena.replace('&#233;','é')
    cadena=cadena.replace('&#228;','ä')
    cadena=cadena.replace('&#214;','Ö')
    cadena=cadena.replace('&amp;','&')
    cadena=cadena.replace('&AMP;','&')
    cadena=cadena.replace('&Ntilde;','Ñ')
    
    return cadena
    
## Converts a decimal to a localized number string
def l10nDecimal(dec, digits=2):
    l=QLocale()
    
    return l.toCurrencyString(float(dec))
    
    
def function_name(clas):
    #    print (inspect.stack()[0][0].f_code.co_name)
    #    print (inspect.stack()[0][3],  inspect.stack())
    #    print (inspect.stack()[1][3],  inspect.stack())
    #    print (clas.__class__.__name__)
    #    print (clas.__module__)
    return "{0}.{1}".format(clas.__class__.__name__,inspect.stack()[1][3])
    
    
#    
#def launchFrmInvestmentReport(mem, high_low, investment, parent):
#    if high_low==False:
#        w=frmInvestmentReport(mem,  investment, parent)
#        w.exec_()
#    else:
#        w=frmInvestmentHlReport(mem, investment, parent)
#        w.exec_()
#    return w

## Check if two numbers has the same sign
## @param number1 First number used in check
## @param number2 Second number used in check
## @return bool True if they have the same sign
def have_same_sign(number1, number2):
    if (is_positive(number1)==True and is_positive(number2)==True) or (is_positive(number1)==False and is_positive(number2)==False):
        return True
    return False

## Check if a number is positive
## @param number Number used in check
## @return bool True if number is positive, else False
def is_positive(number):
    if number>=0:
        return True
    return False

## Sets the sign of other number
def set_sign_of_other_number(number, number_to_change):
    if is_positive(number):
       return abs(number_to_change)
    return -abs(number_to_change)


